import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24301Msgs.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;
import com.objsys.nas.TS24007L3.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24301Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24301Msgs.ActvDfltEPSBearerCtxtReq data = new ActvDfltEPSBearerCtxtReq();
      com.objsys.nas.TS24301Msgs.ActvDfltEPSBearerCtxtReq pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = ProtoDiscr.epsMobMgmt();
      pdu.l3HdrOpts = new L3HdrOptions();
      EPSBearerIdentity tempVar1 = EPSBearerIdentity.valueOf(0);
      pdu.l3HdrOpts.set_epsBearerIdent(tempVar1);

      pdu.protoDiscr = ProtoDiscr.epsSessMgmt();
      pdu.pti = new PDU_pti(PDU_pti._VALUE, new Asn1Integer(0));
      pdu.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_ActvDfltEPSBearerCtxtReq);
      pdu.data = pvalue;
      pvalue.epsQoS = new EPSQualityOfService();
      pvalue.epsQoS.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.epsQoS.qci = new Asn1Integer(205);
      pvalue.accessPointName = new AccessPointName("province1.mnc012.mcc345.gprs");
      pvalue.pdnAddress = new PDNAddress();
      pvalue.pdnAddress.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.pdnAddress.pdnTypeValue = PDNTypeValue.ipv4();
      {
          byte testdata[] = {
         (byte)0x2f, (byte)0x04, (byte)0x32, (byte)0x11 };
         pvalue.pdnAddress.pdnAddrInfo = new Asn1OctetString();
         pvalue.pdnAddress.pdnAddrInfo.value = testdata;
      }
      pvalue.transIdent = new LinkedTI();
      pvalue.transIdent.flag = new Asn1Boolean(false);
      pvalue.transIdent.value = new Asn1Integer(0);
      pvalue.negotiatedQoS = new QualityOfService();
      pvalue.negotiatedQoS.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.negotiatedQoS.spare1 = new Asn1Integer(1);
      pvalue.negotiatedQoS.delayClass = new Asn1Integer(6);
      pvalue.negotiatedQoS.reliabilityClass = new Asn1Integer(6);
      pvalue.negotiatedQoS.peakThroughput = new Asn1Integer(3);
      pvalue.negotiatedQoS.spare2 = new Asn1Integer(0);
      pvalue.negotiatedQoS.precedenceClass = new Asn1Integer(5);
      pvalue.negotiatedQoS.spare3 = new Asn1Integer(6);
      pvalue.negotiatedQoS.meanThroughput = new Asn1Integer(7);
      pvalue.negotiatedQoS.trafficClass = new Asn1Integer(6);
      pvalue.negotiatedQoS.deliveryOrder = new Asn1Integer(2);
      pvalue.negotiatedQoS.deliveryErroneousSDU = new Asn1Integer(1);
      pvalue.negotiatedQoS.maxSDUSize = new Asn1Integer(0);
      pvalue.negotiatedQoS.maxUplinkBR = new Asn1Integer(195);
      pvalue.negotiatedQoS.maxDownlinkBR = new Asn1Integer(198);
      pvalue.negotiatedQoS.residualBER = new Asn1Integer(11);
      pvalue.negotiatedQoS.sduErrorRatio = new Asn1Integer(14);
      pvalue.negotiatedQoS.transferDelay = new Asn1Integer(42);
      pvalue.negotiatedQoS.trafficHandlingPrio = new Asn1Integer(1);
      pvalue.negotiatedQoS.guaranteedUplinkBR = new Asn1Integer(130);
      pvalue.negotiatedQoS.guaranteedDownlinkBR = new Asn1Integer(229);
      pvalue.negotiatedQoS.spare4 = new Asn1Integer(1);
      pvalue.negotiatedQoS.signallingIndication = new Asn1Boolean(true);
      pvalue.negotiatedQoS.sourceStatsDescriptor = new Asn1Integer(1);
      pvalue.negotiatedLLCSAPI = new LLCSAPIValue();
      pvalue.negotiatedLLCSAPI.spare = new Asn1Integer(10);
      pvalue.negotiatedLLCSAPI.value = SAPIValues.sapi5();
      pvalue.radioPriority = new RadioPriority();
      pvalue.radioPriority.value = new Asn1Integer(1);
      pvalue.packetFlowIdentifier = new PacketFlowIdentValue();
      pvalue.packetFlowIdentifier.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.packetFlowIdentifier.spare = new Asn1Boolean(false);
      pvalue.packetFlowIdentifier.value = new Asn1Integer(20);
      pvalue.apn_AMBR = new APNAggMaxBitRate();
      pvalue.apn_AMBR.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.apn_AMBR.apnAmbrforDL = new Asn1Integer(32);
      pvalue.apn_AMBR.apnAmbrforUL = new Asn1Integer(194);

      pvalue.esmCause = ESMCause.operDeterBarring();
      pvalue.protocolConfigOptions = new ProtocolConfigOptions();
      pvalue.protocolConfigOptions.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.protocolConfigOptions.octet3 = new ProtocolConfigOptions_octet3();
      pvalue.protocolConfigOptions.octet3.configProtocol = new Asn1Integer(0);
      pvalue.protocolConfigOptions.octet3.spare = new Asn1Integer(0);
      pvalue.protocolConfigOptions.pcoUnits = new PCOUnit[0];
      pvalue.mV10ExtPresent = true;
      pvalue.connectivityType = new ConnectivityType(11);

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
