import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24301Msgs.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;
import com.objsys.nas.TS24007L3.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24301Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24301Msgs.AttachAccept data = new AttachAccept();
      com.objsys.nas.TS24301Msgs.AttachAccept pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = ProtoDiscr.epsMobMgmt();
      pdu.pti = new PDU_pti(PDU_pti._NOVALUE, null);
      pdu.l3HdrOpts = new L3HdrOptions();
      Asn1Integer tempVar1 = new Asn1Integer(0);
      pdu.l3HdrOpts.set_secHdr(tempVar1);

      pdu.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_AttachAccept);
      pdu.data = pvalue;

      pvalue.spare = new Asn1Integer(8);
      pvalue.result = new EPSAttachResult();
      pvalue.result.spare = new Asn1Boolean(false);
      pvalue.result.result = EPSAttachResultValues.combinedAttach();
      pvalue.t3412Value = new GPRSTimer();
      pvalue.t3412Value.unit = new Asn1Integer(2);
      pvalue.t3412Value.timerValue = new Asn1Integer(24);
      pvalue.taiList = new TrackingAreaIdentList();
      pvalue.taiList.length = new Asn1Integer(94);
      pvalue.taiList.idents = new TrackingAreaIdentList_idents();
      pvalue.taiList.idents.elements = new TrackingAreaIdentListItem[1];
      pvalue.taiList.idents.elements[0] = new TrackingAreaIdentListItem();
      pvalue.taiList.idents.elements[0].spare = new Asn1Boolean(false);
      pvalue.taiList.idents.elements[0].typeOfList = new Asn1Integer(0);
      pvalue.taiList.idents.elements[0].numElements = new Asn1Integer(0); /* +1 */
      pvalue.taiList.idents.elements[0].elements = new TrackingAreaIdentListItem_elements();
      PartialTrkAreaIdent0 tempVar2 = new PartialTrkAreaIdent0();
      pvalue.taiList.idents.elements[0].elements.set_partTrkIdentList0(tempVar2);
      tempVar2.mcc_mnc = new MCCMNCPair();
      tempVar2.mcc_mnc.mcc = new TBCDString3("454");
      tempVar2.mcc_mnc.mnc = new TBCDString3("07");
      tempVar2.tacs = new TrackingAreaCode[1];
      tempVar2.tacs[0] = new TrackingAreaCode();
      {
         byte testdata[] = { (byte)0x41, (byte)0x49 };
         tempVar2.tacs[0].value = testdata;
      }

      com.objsys.nas.TS24301Msgs.PDU_SM pdu_SM = new PDU_SM();
      com.objsys.nas.TS24301Msgs.ActvDfltEPSBearerCtxtReq pvalue_SM = new ActvDfltEPSBearerCtxtReq();

      pdu_SM.l3HdrOpts = new L3HdrOptions();
      pdu_SM.l3HdrOpts.set_epsBearerIdent(EPSBearerIdentity.id5());

      pdu_SM.protoDiscr = ProtoDiscr.epsSessMgmt();
      pdu_SM.pti = new PDU_SM_pti();
      Asn1Integer tempVar4 = new Asn1Integer(0);
      pdu_SM.pti.set_value(tempVar4);

      pdu_SM.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_ActvDfltEPSBearerCtxtReq);
      pdu_SM.data = pvalue_SM;
      pvalue_SM.epsQoS = new EPSQualityOfService();
      pvalue_SM.epsQoS.length = new Asn1Integer(0); /* length will be calculated */
      pvalue_SM.epsQoS.qci = new Asn1Integer(9);
      pvalue_SM.accessPointName = new AccessPointName("uad5gn.au-net.ne.jp.mnc051.mcc440.gprs");
      pvalue_SM.pdnAddress = new PDNAddress();
      pvalue_SM.pdnAddress.length = new Asn1Integer(0); /* length will be calculated */
      pvalue_SM.pdnAddress.pdnTypeValue = PDNTypeValue.ipv4v6();
      {
          byte testdata[] = {
         (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x08, (byte)0x0d, (byte)0x49, (byte)0x97, (byte)0x01, (byte)0x0a, (byte)0xb1, (byte)0x20, (byte)0x70 };
         pvalue_SM.pdnAddress.pdnAddrInfo = new Asn1OctetString();
         pvalue_SM.pdnAddress.pdnAddrInfo.value = testdata;
      }
      pvalue_SM.apn_AMBR = new APNAggMaxBitRate();
      pvalue_SM.apn_AMBR.length = new Asn1Integer(0); /* length will be calculated */
      pvalue_SM.apn_AMBR.apnAmbrforDL = new Asn1Integer(254);
      pvalue_SM.apn_AMBR.apnAmbrforUL = new Asn1Integer(254);
      
      pvalue.esmMessageContainer = new ESMMessageContainer();
      pvalue.esmMessageContainer.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.esmMessageContainer.pduSm = pdu_SM;
      pvalue.guti = new EPSMobileIdentity();
      /* Don't need to set filler or odd/even indicator for GUIT */
      pvalue.guti.typeOfIdent = EPSMobileIdentity_typeOfIdent.guti();
      EPSMobileIdentity_GUTI tempVar5 = new EPSMobileIdentity_GUTI();
      pvalue.guti.content = new EPSMobileIdentity_content();
      
      pvalue.guti.content.set_guti(tempVar5);
      tempVar5.mcc_mnc = new MCCMNCPair();
      tempVar5.mcc_mnc.mcc = new TBCDString3("454");
      tempVar5.mcc_mnc.mnc = new TBCDString3("07");
      {
         byte testdata[] = {
            (byte)0x1b, (byte)0x38 };

         tempVar5.mmeGroupID = new Asn1OctetString(testdata);
      }

      tempVar5.mmeCode = new Asn1Integer(251);
      {
          byte testdata[] = {
         (byte)0xd9, (byte)0x1f, (byte)0xe5, (byte)0xf8 };
         tempVar5.tmsi = new Asn1OctetString(testdata);
      }

      pvalue.locAreaIdent = new LocAreaIdent();
      pvalue.locAreaIdent.mcc_mnc = new MCCMNCPair();
      pvalue.locAreaIdent.mcc_mnc.mcc = new TBCDString3("454");
      pvalue.locAreaIdent.mcc_mnc.mnc = new TBCDString3("07");
      pvalue.locAreaIdent.lac = new Asn1Integer(23163);
      pvalue.msIdentity = new MobileIdentity();
      pvalue.msIdentity.set_imei(new IMEI( "050b8*23870*6c2" ));
      pvalue.emmCause = EMMCause.imsiUnkInHSS();
      pvalue.t3402 = new GPRSTimer();
      pvalue.t3402.unit = new Asn1Integer(6);
      pvalue.t3402.timerValue = new Asn1Integer(26);
      pvalue.t3423 = new GPRSTimer();
      pvalue.t3423.unit = new Asn1Integer(0);
      pvalue.t3423.timerValue = new Asn1Integer(3);
      pvalue.equivPLMNs = new PLMNList();
      pvalue.equivPLMNs.elements = new MCCMNCPair[1];
      pvalue.equivPLMNs.elements[0] = new MCCMNCPair();
      pvalue.equivPLMNs.elements[0].mcc = new TBCDString3("405");
      pvalue.equivPLMNs.elements[0].mnc = new TBCDString3("025");
      pvalue.emergNumList = new EmergencyNumberList();
      pvalue.emergNumList.length = new Asn1Integer(0); /* will be calculated */
      pvalue.emergNumList.numbers = new EmergencyNumberList_numbers();
      pvalue.emergNumList.numbers.elements = new EmergencyNumber[1];
      pvalue.emergNumList.numbers.elements[0] = new EmergencyNumber();
      pvalue.emergNumList.numbers.elements[0].length = new Asn1Integer(0);
      pvalue.emergNumList.numbers.elements[0].spare = new Asn1Integer(0);
      pvalue.emergNumList.numbers.elements[0].catValue = new Asn1Integer(10);
      pvalue.emergNumList.numbers.elements[0].digits = new TBCDString("5c");
      pvalue.epsNetFeatSupp = new EPSNetworkFeatureSupport();
      pvalue.epsNetFeatSupp.length = new Asn1Integer(1);
      pvalue.epsNetFeatSupp.cpCIoT = new Asn1Boolean(false);
      pvalue.epsNetFeatSupp.erNoPDN = new Asn1Boolean(true);
      pvalue.epsNetFeatSupp.esr = new Asn1Boolean(false);
      pvalue.epsNetFeatSupp.csLCS = new Asn1Integer(2);
      pvalue.epsNetFeatSupp.epcLCS = new Asn1Boolean(false);
      pvalue.epsNetFeatSupp.emcBS = new Asn1Boolean(false);
      pvalue.epsNetFeatSupp.imsVoPS = new Asn1Boolean(true);
      pvalue.addnUpdateResult = new AddnUpdateResult();
      pvalue.addnUpdateResult.spare = new Asn1Integer(0);
      pvalue.addnUpdateResult.value = AddnUpdateResult_value.noInfo();

      pvalue.mV17ExtPresent = true;
      pvalue.negotiatedImsiOffset = new IMSIOffset();
      pvalue.negotiatedImsiOffset.value = new Asn1OctetString();
      pvalue.negotiatedImsiOffset.value.value = new byte[2];
      pvalue.negotiatedImsiOffset.value.value[0] = (byte)0xab;
      pvalue.negotiatedImsiOffset.value.value[1] = (byte)0xcd;

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
