import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24301Msgs.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;
import com.objsys.nas.TS24007L3.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24301Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24301Msgs.AuthFailure data = new AuthFailure();
      com.objsys.nas.TS24301Msgs.AuthFailure pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = ProtoDiscr.epsMobMgmt();
      pdu.pti = new PDU_pti(PDU_pti._NOVALUE, null);      
      pdu.l3HdrOpts = new L3HdrOptions();
      Asn1Integer tempVar1 = new Asn1Integer(0);
      pdu.l3HdrOpts.set_secHdr(tempVar1);

      pdu.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_AuthFailure);
      pdu.data = pvalue;
      pvalue.emmCause = EMMCause.imsiUnkInHSS();
      pvalue.authFailureParam = new AuthFailureParam();
      {
          byte testdata[] = {
         (byte)0xd9, (byte)0xa8, (byte)0x3b, (byte)0x16, (byte)0xef, (byte)0x11, (byte)0x91, (byte)0xd5,      (byte)0xa7, (byte)0x3d, (byte)0xdc, (byte)0x1e, (byte)0xf0, 0xd };
         pvalue.authFailureParam.param = new Asn1OctetString(testdata);
      }

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
