import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24301Msgs.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;
import com.objsys.nas.TS24007L3.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24301Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24301Msgs.BearerResAllocReq data = new BearerResAllocReq();
      com.objsys.nas.TS24301Msgs.BearerResAllocReq pvalue = data;

      /* Populate data structure */
      pdu.l3HdrOpts = new L3HdrOptions();
      pdu.l3HdrOpts.set_epsBearerIdent(EPSBearerIdentity.noIdent());

      pdu.protoDiscr = ProtoDiscr.epsSessMgmt();
      pdu.pti = new PDU_pti(PDU_pti._VALUE, new Asn1Integer(0));
      pdu.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_BearerResAllocReq);
      pdu.data = pvalue;
      pvalue.spare = new Asn1Integer(13);
      pvalue.linkedEPSBearerIdent = new LinkedEPSBearerIdent(14);
      pvalue.trafficFlowAgg = new TFT();
      pvalue.trafficFlowAgg.length = new Asn1Integer(216);
      pvalue.trafficFlowAgg.octet1 = new TFTOctet1();
      pvalue.trafficFlowAgg.octet1.tftOperationCode = TFTOctet1_tftOperationCode.delPktFilters();
      pvalue.trafficFlowAgg.octet1.eBit = new Asn1Boolean(false);
      pvalue.trafficFlowAgg.octet1.numPacketFilters = new Asn1Integer(15);
      pvalue.trafficFlowAgg.packetFilterList = new TFT_packetFilterList();
      Asn1OctetString tempVar2 = new Asn1OctetString();
      pvalue.trafficFlowAgg.packetFilterList.set_identifiers(tempVar2);
      {
          byte testdata[] = {
         (byte)0xbd, (byte)0x64, (byte)0x2b, (byte)0x52, (byte)0x26, (byte)0x63,
         (byte)0x03, (byte)0xd8, (byte)0x4f, (byte)0xfc, (byte)0x1f, (byte)0x6a,
         (byte)0x98, (byte)0xa3, (byte)0x26 };
         tempVar2.value = testdata;
      }
      pvalue.epsQoS = new EPSQualityOfService();
      pvalue.epsQoS.length = new Asn1Integer(1);
      pvalue.epsQoS.qci = new Asn1Integer(14);
      pvalue.protocolConfigOptions = new ProtocolConfigOptions();
      pvalue.protocolConfigOptions.length = new Asn1Integer(248);
      pvalue.protocolConfigOptions.octet3 = new ProtocolConfigOptions_octet3();
      pvalue.protocolConfigOptions.octet3.spare = new Asn1Integer(7);
      pvalue.protocolConfigOptions.octet3.configProtocol = new Asn1Integer(1);
      pvalue.mV10ExtPresent = true;
      pvalue.deviceProperties = new DeviceProperties();
      pvalue.deviceProperties.spare = new Asn1Integer(2);
      pvalue.deviceProperties.lowPriority = new Asn1Boolean(false);

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
