import com.objsys.asn1j.runtime.*;
import java.io.*;

public class Reader {
   private static final String JSONFILE = "message.json";
   private static final String XERFILE = "message.xml";
   private static boolean trace = true;

   public static void main (String args[]) {
      String filename = new String ("message.dat");

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
               Diag.instance().setEnabled (true);
            else if (args[i].equals ("-i")) 
               filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
               System.out.println ("usage: Reader [ -v ] [ -i <filename>");
               System.out.println ("   -v  verbose mode: print trace info");
               System.out.println ("   -i <filename>  " +
                                   "read encoded msg from <filename>");
               System.out.println ("   -notrace  do not display trace info");
               return;
            }
         }
      }

      try {
         /* Round-trip the NAS data file through JER and XER.  This should
            work and the end result should be the same NAS data as input.
         */
         Asn1NasDecodeBuffer decodeBuffer = roundtrip(filename);
         
          // Enable bit field tracing

         if (trace) {
            decodeBuffer.getTraceHandler().enable();
         }

         // Read and decode the message
         com.objsys.nas.TS24301Msgs.PDU pdu = new com.objsys.nas.TS24301Msgs.PDU();
         try {            
            pdu.decode (decodeBuffer);
            if ( trace ) System.out.println ("Decode was successful");
         }
         catch ( Exception e ) 
         {
            if ( trace ) System.out.println ("Decode failed.");
            throw e;
         }
         finally {
            if (trace) {               
               pdu.print (System.out, "pdu", 0);
               System.out.println ("");
               System.out.println ("Binary trace:");
               decodeBuffer.binDump ("pdu");
            }
         }
      }
      catch (Exception e) {
         System.out.println (e.getMessage());
         e.printStackTrace();           
         System.exit(-1);
      }
   }
   
   /**
    * Roundtrip the given NAS file through JER and XER, and return an
    * AsnNasDecodeBuffer on the roundtripped data.
    */
   public static Asn1NasDecodeBuffer roundtrip(String filename)
      throws Exception 
   {
      // Create an input file stream object

      FileInputStream in = new FileInputStream (filename);

       // Create a decode buffer object

      Asn1NasDecodeBuffer decodeBuffer = 
         new Asn1NasDecodeBuffer (in);
         
      decodeBuffer.mDirection = Asn1NasDecodeBuffer.UPLINK;
      
      com.objsys.nas.TS24301Msgs.PDU pdu = new com.objsys.nas.TS24301Msgs.PDU();
      pdu.decode (decodeBuffer);
         
      encodeToJSONFile(pdu);
      pdu = decodeFromJSONFile();
      encodeToXERFile(pdu);
      pdu = decodeFromXERFile();

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();       
      pdu.encode(encodeBuffer);
      
      decodeBuffer = new Asn1NasDecodeBuffer(encodeBuffer.getByteArrayInputStream());
      decodeBuffer.mDirection = Asn1NasDecodeBuffer.UPLINK;
      return decodeBuffer;
   }
    
   /**
    * Encode struct to JSON file.
    */
   public static void encodeToJSONFile(com.objsys.nas.TS24301Msgs.PDU pdu)
      throws Exception
   {
      // Create an encoding stream
      Asn1JsonOutputStream encodeStream = null;
      try {
         encodeStream = new Asn1JsonOutputStream (
                           new java.io.OutputStreamWriter( 
                              new java.io.FileOutputStream(JSONFILE),
                              "UTF-8") );

         pdu.encode (encodeStream);
         if (trace) {
            System.out.println("Encoding to JSON was successful");
         }
      }
      finally {
         try {
            if (encodeStream != null) encodeStream.close ();
         }
         catch (Exception e) {
         }
      }      
   }
   
   /**
    * Decode JSON file to struct.
    */
   public static com.objsys.nas.TS24301Msgs.PDU decodeFromJSONFile()
      throws Exception
   {
      // Create an input file stream object
      java.io.FileInputStream ins = new java.io.FileInputStream (JSONFILE);

      Asn1JsonDecodeBuffer decodeBuffer = new Asn1JsonDecodeBuffer(ins);
      // Read and decode the message
      com.objsys.nas.TS24301Msgs.PDU value = new com.objsys.nas.TS24301Msgs.PDU ();
      value.decode (decodeBuffer);
      if (trace) {
         System.out.println("Decode JSON was successful");
      }
      
      return value;
   }
   
   /**
    * Encode struct to XER file.
    */
   public static void encodeToXERFile(com.objsys.nas.TS24301Msgs.PDU pdu)
      throws Exception
   {
      // Create a message buffer object and encode the record
      boolean cxer = false;
      
      Asn1XerEncodeBuffer encodeBuffer = new Asn1XerEncodeBuffer(cxer, 0);

      encodeBuffer.encodeStartDocument ();

      pdu.encode (encodeBuffer, null);

      encodeBuffer.encodeEndDocument ();

      if (trace) {
         System.out.println ("Encoding to XER was successful");
      }

      // Write the encoded record to a file

      encodeBuffer.write (new FileOutputStream (XERFILE));
   }
   
   /**
    * Decode XER file to struct.
    */
   public static com.objsys.nas.TS24301Msgs.PDU decodeFromXERFile()
      throws Exception
   {
      java.io.InputStream inputFile = null;

      try {
         // Create an XML reader object

         inputFile = new java.io.BufferedInputStream(
                        new java.io.FileInputStream(XERFILE) );

         org.xmlpull.v1.XmlPullParserFactory xmlInputFactory =
             	org.xmlpull.v1.XmlPullParserFactory.newInstance();

         org.xmlpull.v1.XmlPullParser reader =
            xmlInputFactory.newPullParser();

         reader.setFeature(
            org.xmlpull.v1.XmlPullParser.FEATURE_PROCESS_NAMESPACES, true);

         reader.setInput(inputFile, null);


         // Read and decode the message

         com.objsys.nas.TS24301Msgs.PDU pdu = new com.objsys.nas.TS24301Msgs.PDU ();
         pdu.decodeDocument(reader);
         if (trace) {
            System.out.println ("Decode XER was successful");
         }
         
         return pdu;
      }
      finally {
         if ( inputFile != null )
            try { inputFile.close(); } catch (java.io.IOException e) {}
      }
   }    
}
