import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24301Msgs.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;
import com.objsys.nas.TS24007L3.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24301Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24301Msgs.EMMInformation data = new EMMInformation();
      com.objsys.nas.TS24301Msgs.EMMInformation pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = ProtoDiscr.epsMobMgmt();
      pdu.pti = new PDU_pti(PDU_pti._NOVALUE, null);
      pdu.l3HdrOpts = new L3HdrOptions();
      Asn1Integer tempVar1 = new Asn1Integer(0);
      pdu.l3HdrOpts.set_secHdr(tempVar1);
      pdu.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_EMMInformation);
      pdu.data = pvalue;
      pvalue.fullNameForNetwork = new NetworkName();
      pvalue.fullNameForNetwork.length = new Asn1Integer(186);
      pvalue.fullNameForNetwork.ext = new Asn1Boolean(false);
      pvalue.fullNameForNetwork.codingScheme = NetworkName_codingScheme.callBcastData();
      pvalue.fullNameForNetwork.addCI = new Asn1Boolean(false);
      pvalue.fullNameForNetwork.textString = new NetworkName_textString();
      GSMString tempVar2 = new GSMString("cqRuq2g");
      pvalue.fullNameForNetwork.textString.set_callBcastData(tempVar2);
      pvalue.fullNameForNetwork.numSpareBitsInLastOct = new Asn1Integer(tempVar2.value.length() % 8);
      pvalue.shortNameForNetwork = new NetworkName();
      pvalue.shortNameForNetwork.length = new Asn1Integer(206);
      pvalue.shortNameForNetwork.ext = new Asn1Boolean(true);
      pvalue.shortNameForNetwork.codingScheme = NetworkName_codingScheme.callBcastData();
      pvalue.shortNameForNetwork.addCI = new Asn1Boolean(true);
      pvalue.shortNameForNetwork.numSpareBitsInLastOct = new Asn1Integer(0);
      pvalue.shortNameForNetwork.textString = new NetworkName_textString();
      GSMString tempVar3 = new GSMString("OP@@S3");
      pvalue.shortNameForNetwork.textString.set_callBcastData(tempVar3);
      pvalue.shortNameForNetwork.numSpareBitsInLastOct = new Asn1Integer(tempVar3.value.length() % 8);
      pvalue.localTimeZone = new TimeZone(33);
      pvalue.univTimeAndLclTimeZone = new TimeZoneAndTime();
      pvalue.univTimeAndLclTimeZone.year = new Asn1Integer(216);
      pvalue.univTimeAndLclTimeZone.month = new Asn1Integer(183);
      pvalue.univTimeAndLclTimeZone.day = new Asn1Integer(99);
      pvalue.univTimeAndLclTimeZone.hour = new Asn1Integer(67);
      pvalue.univTimeAndLclTimeZone.minute = new Asn1Integer(142);
      pvalue.univTimeAndLclTimeZone.second = new Asn1Integer(104);
      pvalue.univTimeAndLclTimeZone.timezone = new Asn1Integer(231);
      pvalue.networkDaylightSavingTime = new DaylightSavingTime();
      pvalue.networkDaylightSavingTime.spare = new Asn1Integer(3);
      pvalue.networkDaylightSavingTime.dst = DaylightSavingTime_dst.dstNoAdjust();

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
