import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24301Msgs.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;
import com.objsys.nas.TS24007L3.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24301Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24301Msgs.GUTIReallocCmd data = new GUTIReallocCmd();
      com.objsys.nas.TS24301Msgs.GUTIReallocCmd pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = ProtoDiscr.epsMobMgmt();
      pdu.pti = new PDU_pti(PDU_pti._NOVALUE, null);
      pdu.l3HdrOpts = new L3HdrOptions();
      Asn1Integer tempVar1 = new Asn1Integer(0);
      pdu.l3HdrOpts.set_secHdr(tempVar1);

      pdu.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_GUTIReallocCmd);
      pdu.data = pvalue;
      pvalue.guti = new EPSMobileIdentity();
      pvalue.guti.length = new Asn1Integer(0); /* will be calculated */
      pvalue.guti.typeOfIdent = EPSMobileIdentity_typeOfIdent.imsi();
      pvalue.guti.content = new EPSMobileIdentity_content();
      pvalue.guti.content.set_imsi(new IMSI("48487598410102"));
      pvalue.taiList = new TrackingAreaIdentList();
      pvalue.taiList.length = new Asn1Integer(0);
      pvalue.taiList.idents = new TrackingAreaIdentList_idents();
      pvalue.taiList.idents.elements = new TrackingAreaIdentListItem[1];
      pvalue.taiList.idents.elements[0] = new TrackingAreaIdentListItem();
      pvalue.taiList.idents.elements[0].spare = new Asn1Boolean(false);
      pvalue.taiList.idents.elements[0].typeOfList = new Asn1Integer(1);
      pvalue.taiList.idents.elements[0].numElements = new Asn1Integer(9);
      pvalue.taiList.idents.elements[0].elements = new TrackingAreaIdentListItem_elements();
      PartialTrkAreaIdent1 tempVar3 = new PartialTrkAreaIdent1();
      pvalue.taiList.idents.elements[0].elements.set_partTrkIdentList1(tempVar3);

      tempVar3.mcc_mnc = new MCCMNCPair();
      tempVar3.mcc_mnc.mcc = new TBCDString3("646");
      tempVar3.mcc_mnc.mnc = new TBCDString3("01");
      {
          byte testdata[] = {
         (byte)0x11, (byte)0x79 };
         tempVar3.tac = new TrackingAreaCode();
         tempVar3.tac.value = testdata;
      }

      pvalue.mV16ExtPresent = true;
      pvalue.ueRadioCapId = new UERadioCapId();
      pvalue.ueRadioCapId.capId = new Asn1OctetString();
      {
          byte testdata[] = { (byte)0xab, (byte)0xcd };
         pvalue.ueRadioCapId.capId.value = testdata;
      }
      pvalue.ueRadioCapIdDelInd = new UERadioCapIdDeletionInd();
      pvalue.ueRadioCapIdDelInd.deletionReq = new Asn1Integer(5);

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
