import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24301Msgs.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;
import com.objsys.nas.TS24007L3.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24301Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24301Msgs.ModEPSBearerCtxtReq data = new ModEPSBearerCtxtReq();
      com.objsys.nas.TS24301Msgs.ModEPSBearerCtxtReq pvalue = data;

      /* Populate data structure */
      pdu.l3HdrOpts = new L3HdrOptions();      
      pdu.l3HdrOpts.set_epsBearerIdent(EPSBearerIdentity.noIdent());
      pdu.protoDiscr = ProtoDiscr.epsSessMgmt();
      pdu.pti = new PDU_pti(PDU_pti._VALUE, new Asn1Integer(0));
      pdu.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_ModEPSBearerCtxtReq);
      pdu.data = pvalue;
      pvalue.newEPSQoS = new EPSQualityOfService();
      pvalue.newEPSQoS.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.newEPSQoS.qci = new Asn1Integer(181);
      pvalue.tft = new TFT();
      pvalue.tft.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.tft.octet1 = new TFTOctet1();
      pvalue.tft.octet1.tftOperationCode = TFTOctet1_tftOperationCode.delPktFilters();
      pvalue.tft.octet1.eBit = new Asn1Boolean(true);
      pvalue.tft.octet1.numPacketFilters = new Asn1Integer(15);
      pvalue.tft.packetFilterList = new TFT_packetFilterList();
      Asn1OctetString tempVar2 = new Asn1OctetString();
      pvalue.tft.packetFilterList.set_identifiers(tempVar2);
      {
          byte testdata[] = {
         (byte)0x3f, (byte)0x92, (byte)0x49, (byte)0xd2, (byte)0xa5, (byte)0xc5,
         (byte)0xa1, (byte)0x02, (byte)0xce, (byte)0x85, (byte)0x1c, (byte)0x9f,
         (byte)0x8d, (byte)0x2a, (byte)0x74 };
         tempVar2.value = testdata;
      }
      pvalue.newQoS = new QualityOfService();
      pvalue.newQoS.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.newQoS.spare1 = new Asn1Integer(0);
      pvalue.newQoS.delayClass = new Asn1Integer(0);
      pvalue.newQoS.reliabilityClass = new Asn1Integer(1);
      pvalue.newQoS.peakThroughput = new Asn1Integer(12);
      pvalue.newQoS.spare2 = new Asn1Integer(0);
      pvalue.newQoS.precedenceClass = new Asn1Integer(4);
      pvalue.newQoS.spare3 = new Asn1Integer(4);
      pvalue.newQoS.meanThroughput = new Asn1Integer(30);
      pvalue.newQoS.trafficClass = new Asn1Integer(1);
      pvalue.newQoS.deliveryOrder = new Asn1Integer(0);
      pvalue.newQoS.deliveryErroneousSDU = new Asn1Integer(5);
      pvalue.newQoS.maxSDUSize = new Asn1Integer(246);
      pvalue.newQoS.maxUplinkBR = new Asn1Integer(194);
      pvalue.newQoS.maxDownlinkBR = new Asn1Integer(197);
      pvalue.newQoS.residualBER = new Asn1Integer(4);
      pvalue.newQoS.sduErrorRatio = new Asn1Integer(2);
      pvalue.newQoS.transferDelay = new Asn1Integer(28);
      pvalue.newQoS.trafficHandlingPrio = new Asn1Integer(0);
      pvalue.newQoS.guaranteedUplinkBR = new Asn1Integer(225);
      pvalue.newQoS.guaranteedDownlinkBR = new Asn1Integer(91);
      pvalue.newQoS.spare4 = new Asn1Integer(6);
      pvalue.newQoS.signallingIndication = new Asn1Boolean(true);
      pvalue.newQoS.sourceStatsDescriptor = new Asn1Integer(7);
      pvalue.negotiatedLLCSAPI = new LLCSAPIValue();
      pvalue.negotiatedLLCSAPI.spare = new Asn1Integer(4);
      pvalue.negotiatedLLCSAPI.value = SAPIValues.sapi9();
      pvalue.radioPriority = new RadioPriority();
      pvalue.radioPriority.value = new Asn1Integer(1);
      pvalue.packetFlowIdentifier = new PacketFlowIdentValue();
      pvalue.packetFlowIdentifier.length = new Asn1Integer(0);  /* length will be calculated */
      pvalue.packetFlowIdentifier.spare = new Asn1Boolean(false);
      pvalue.packetFlowIdentifier.value = new Asn1Integer(107);
      pvalue.apn_AMBR = new APNAggMaxBitRate();
      pvalue.apn_AMBR.length = new Asn1Integer(4);
      pvalue.apn_AMBR.apnAmbrforDL = new Asn1Integer(203);
      pvalue.apn_AMBR.apnAmbrforUL = new Asn1Integer(208);
      pvalue.protocolConfigOptions = new ProtocolConfigOptions();
      pvalue.protocolConfigOptions.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.protocolConfigOptions.octet3 = new ProtocolConfigOptions_octet3();
      pvalue.protocolConfigOptions.octet3.configProtocol = new Asn1Integer(0);
      pvalue.protocolConfigOptions.octet3.spare = new Asn1Integer(0);
      pvalue.protocolConfigOptions.pcoUnits = new PCOUnit[0];

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
