import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24301Msgs.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;
import com.objsys.nas.TS24007L3.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24301Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24301Msgs.PDNConnectReq data = new PDNConnectReq();
      com.objsys.nas.TS24301Msgs.PDNConnectReq pvalue = data;

      /* Populate data structure */
      pdu.l3HdrOpts = new L3HdrOptions();
      pdu.l3HdrOpts.set_epsBearerIdent(EPSBearerIdentity.noIdent());
      pdu.protoDiscr = ProtoDiscr.epsSessMgmt();
      pdu.pti = new PDU_pti(PDU_pti._VALUE, new Asn1Integer(0));
      pdu.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_PDNConnectReq);
      pdu.data = pvalue;
      pvalue.requestType = new RequestTypeValue();
      pvalue.requestType.spare = new Asn1Boolean(false);
      pvalue.requestType.value = ReqTypeValues.emergency();
      pvalue.pdnType = new PDNType();
      pvalue.pdnType.spare = new Asn1Boolean(false);
      pvalue.pdnType.pdnTypeValue = PDNTypeValue.ipv4();
      pvalue.esmInfoXferFlag = new ESMInfoXferFlag();
      pvalue.esmInfoXferFlag.eitValue = new Asn1Boolean(true);
      pvalue.accessPointName = new AccessPointName("province1.mnc012.mcc345.gprs");
      pvalue.protocolConfigOptions = new ProtocolConfigOptions();
      pvalue.protocolConfigOptions.length = new Asn1Integer(0); // Calculated automatically...
      pvalue.protocolConfigOptions.octet3 = new ProtocolConfigOptions_octet3();
      pvalue.protocolConfigOptions.octet3.configProtocol = new Asn1Integer(0);
      pvalue.mV10ExtPresent = true;
      pvalue.deviceProperties = new DeviceProperties();
      pvalue.deviceProperties.lowPriority = new Asn1Boolean(false);
      pvalue.mV13ExtPresent = true;
      pvalue.hdrCompressConfig = new HdrCompressConfig();
      pvalue.hdrCompressConfig.profile104 = new Asn1Boolean(true);
      pvalue.hdrCompressConfig.profile103 = new Asn1Boolean(false);
      pvalue.hdrCompressConfig.profile102 = new Asn1Boolean(true);
      pvalue.hdrCompressConfig.profile006 = new Asn1Boolean(false);
      pvalue.hdrCompressConfig.profile004 = new Asn1Boolean(true);
      pvalue.hdrCompressConfig.profile003 = new Asn1Boolean(false);
      pvalue.hdrCompressConfig.profile002 = new Asn1Boolean(true);
      pvalue.hdrCompressConfig.maxCID = new Asn1OctetString();
      pvalue.hdrCompressConfig.maxCID.value = new byte[2];
      pvalue.hdrCompressConfig.maxCID.value[0] = (byte)0x3c;
      pvalue.hdrCompressConfig.maxCID.value[1] = (byte)0xdc;
      pvalue.hdrCompressConfig.addlSetupType = HdrCompressCtxtParams.esp_ip_3();
      pvalue.hdrCompressConfig.addlSetupContainer = new Asn1OctetString();
      {
          byte testdata[] = { (byte)0xaa, (byte)0xbb, (byte)0xee, (byte)0x11 };
         pvalue.hdrCompressConfig.addlSetupContainer.value = testdata;
      }
      
      // Create a message buffer object and encode the record
      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
