import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24301Msgs.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;
import com.objsys.nas.TS24007L3.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24301Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24301Msgs.RemoteUEReport data = new RemoteUEReport();
      com.objsys.nas.TS24301Msgs.RemoteUEReport pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = ProtoDiscr.epsSessMgmt();
      pdu.pti = new PDU_pti(PDU_pti._VALUE, new Asn1Integer(0));
      pdu.l3HdrOpts = new L3HdrOptions();
      pdu.l3HdrOpts.set_epsBearerIdent(EPSBearerIdentity.id5());
      pdu.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_RemoteUEReport);
      pdu.data = pvalue;
      pvalue.remoteUECtxtConn = new RemoteUECtxtList();
      pvalue.remoteUECtxtConn.numRemUECtxts = new Asn1Integer(1);
      pvalue.remoteUECtxtConn.remUECtxts = new RemoteUECtxt[1];
      pvalue.remoteUECtxtConn.remUECtxts[0] = new RemoteUECtxt();
      pvalue.remoteUECtxtConn.remUECtxts[0].length = new Asn1Integer(0);
      pvalue.remoteUECtxtConn.remUECtxts[0].numUserIdents = new Asn1Integer(1);
      pvalue.remoteUECtxtConn.remUECtxts[0].userIdents = new UserIdentity[1];
      pvalue.remoteUECtxtConn.remUECtxts[0].userIdents[0] = new UserIdentity();
      pvalue.remoteUECtxtConn.remUECtxts[0].userIdents[0].length = new Asn1Integer(0);
      pvalue.remoteUECtxtConn.remUECtxts[0].userIdents[0].userDigit1 = new Asn1Integer(10);
      pvalue.remoteUECtxtConn.remUECtxts[0].userIdents[0].oddEvenInd = new Asn1Boolean(true);
      pvalue.remoteUECtxtConn.remUECtxts[0].userIdents[0].userType1 = UserIdentityType.imeisv();
      pvalue.remoteUECtxtConn.remUECtxts[0].userIdents[0].userDigits = new Asn1OctetString();
      {
          byte testdata[] = { (byte)0xc4, (byte)0x1b };
         pvalue.remoteUECtxtConn.remUECtxts[0].userIdents[0].userDigits.value = testdata;
      }
      pvalue.remoteUECtxtConn.remUECtxts[0].upri4 = new Asn1Boolean(false);
      pvalue.remoteUECtxtConn.remUECtxts[0].tpri4 = new Asn1Boolean(false);
      pvalue.remoteUECtxtConn.remUECtxts[0].addrType = RemoteUEAddrType.remUEIpv4();
      pvalue.remoteUECtxtConn.remUECtxts[0].addrInfo = new Asn1OctetString();
      {
          byte testdata[] = { (byte)0xc3, (byte)0xb1, (byte)0x8e, (byte)0x71, (byte)0x9a, (byte)0x8b };
         pvalue.remoteUECtxtConn.remUECtxts[0].addrInfo.value = testdata;
      }
      pvalue.proSeKeyMgmtFunc = new PKMFAddress();
      pvalue.proSeKeyMgmtFunc.addrType = PKMFAddress_addrType.ipv4();
      pvalue.proSeKeyMgmtFunc.addrInfo = new Asn1OctetString();
      pvalue.proSeKeyMgmtFunc.addrInfo.value = new byte[3];
      pvalue.proSeKeyMgmtFunc.addrInfo.value[0] = (byte)0x7e;
      pvalue.proSeKeyMgmtFunc.addrInfo.value[1] = (byte)0x44;
      pvalue.proSeKeyMgmtFunc.addrInfo.value[2] = (byte)0x98;

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
