import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24301Msgs.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;
import com.objsys.nas.TS24007L3.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24301Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24301Msgs.SecModeCmd data = new SecModeCmd();
      com.objsys.nas.TS24301Msgs.SecModeCmd pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = ProtoDiscr.epsMobMgmt();
      pdu.pti = new PDU_pti(PDU_pti._NOVALUE, null);
      pdu.l3HdrOpts = new L3HdrOptions();
      Asn1Integer tempVar1 = new Asn1Integer(0);
      pdu.l3HdrOpts.set_secHdr(tempVar1);
      pdu.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_SecModeCmd);
      pdu.data = pvalue;
      
      pvalue.selNASSecurityAlgs = new NASSecurityAlgorithms();
      pvalue.selNASSecurityAlgs.spare1 = new Asn1Boolean(false);
      pvalue.selNASSecurityAlgs.typeOfCiph = NASSecurityAlgorithms_typeOfCiph.eea0();
      pvalue.selNASSecurityAlgs.spare2 = new Asn1Boolean(true);
      pvalue.selNASSecurityAlgs.typeOfProt = NASSecurityAlgorithms_typeOfProt.eia0();
      pvalue.spare = new Asn1Integer(4);
      pvalue.nasKeySetIdentifier = new NASKeySetIdentifier();
      pvalue.nasKeySetIdentifier.tsc = new Asn1Boolean(false);
      pvalue.nasKeySetIdentifier.ident = new Asn1Integer(0);
      pvalue.replayedUESecurityCapabilities = new UESecurityCapability();
      pvalue.replayedUESecurityCapabilities.length = new Asn1Integer(0);
      pvalue.replayedUESecurityCapabilities.epsEncryptAlgs = new EPSEncryptAlgsSupported();
      pvalue.replayedUESecurityCapabilities.epsEncryptAlgs.eea0 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.epsEncryptAlgs.x128_eea1 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.epsEncryptAlgs.x128_eea2 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.epsEncryptAlgs.eea3 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.epsEncryptAlgs.eea4 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.epsEncryptAlgs.eea5 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.epsEncryptAlgs.eea6 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.epsEncryptAlgs.eea7 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.epsIntegAlgs = new EPSIntegAlgsSupported();
      pvalue.replayedUESecurityCapabilities.epsIntegAlgs.eia0 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.epsIntegAlgs.eia1 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.epsIntegAlgs.eia2 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.epsIntegAlgs.eia3 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.epsIntegAlgs.eia4 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.epsIntegAlgs.eia5 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.epsIntegAlgs.eia6 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.epsIntegAlgs.eia7 = new Asn1Boolean(true);

      pvalue.replayedUESecurityCapabilities.umtsEncryptAlgs = new UMTSEncryptAlgsSupported();
      pvalue.replayedUESecurityCapabilities.umtsEncryptAlgs.uea0 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.umtsEncryptAlgs.uea1 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.umtsEncryptAlgs.uea2 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.umtsEncryptAlgs.uea3 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.umtsEncryptAlgs.uea4 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.umtsEncryptAlgs.uea5 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.umtsEncryptAlgs.uea6 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.umtsEncryptAlgs.uea7 = new Asn1Boolean(false);

      pvalue.replayedUESecurityCapabilities.octet6 = new UESecurityCapability_octet6();
      pvalue.replayedUESecurityCapabilities.octet6.spare = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.octet6.umtsIntegAlgs = new UMTSIntegAlgsSupported();
      pvalue.replayedUESecurityCapabilities.octet6.umtsIntegAlgs.uia1 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.octet6.umtsIntegAlgs.uia2 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.octet6.umtsIntegAlgs.uia3 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.octet6.umtsIntegAlgs.uia4 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.octet6.umtsIntegAlgs.uia5 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.octet6.umtsIntegAlgs.uia6 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.octet6.umtsIntegAlgs.uia7 = new Asn1Boolean(true);

      pvalue.replayedUESecurityCapabilities.octet7 = new UESecurityCapability_octet7();
      pvalue.replayedUESecurityCapabilities.octet7.spare = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.octet7.gprsIntegAlgs = new GPRSIntegAlgsSupported();
      pvalue.replayedUESecurityCapabilities.octet7.gprsIntegAlgs.gea1 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.octet7.gprsIntegAlgs.gea2 = new Asn1Boolean(false);
      pvalue.replayedUESecurityCapabilities.octet7.gprsIntegAlgs.gea3 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.octet7.gprsIntegAlgs.gea4 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.octet7.gprsIntegAlgs.gea5 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.octet7.gprsIntegAlgs.gea6 = new Asn1Boolean(true);
      pvalue.replayedUESecurityCapabilities.octet7.gprsIntegAlgs.gea7 = new Asn1Boolean(true);

      pvalue.imeisvRequest = new IMEISVRequest();
      pvalue.imeisvRequest.spare = new Asn1Boolean(false);
      pvalue.imeisvRequest.value = new Asn1Integer(5);
      {
          byte testdata[] = {
         (byte)0xc9, (byte)0x80, (byte)0x0a, (byte)0x21 };
         pvalue.replayedNonce = new Nonce();
         pvalue.replayedNonce.value = testdata;
      }
      {
          byte testdata[] = {
         (byte)0x44, (byte)0xcd, (byte)0x17, (byte)0x10 };
         pvalue.nonce = new Nonce();
         pvalue.nonce.value = testdata;
      }

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
