import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24301Msgs.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;
import com.objsys.nas.TS24007L3.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24301Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24301Msgs.TrkAreaUpdRequest data = new TrkAreaUpdRequest();
      com.objsys.nas.TS24301Msgs.TrkAreaUpdRequest pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = ProtoDiscr.epsMobMgmt();
      pdu.pti = new PDU_pti(PDU_pti._NOVALUE, null);
      pdu.l3HdrOpts = new L3HdrOptions();
      Asn1Integer tempVar1 = new Asn1Integer(0);
      pdu.l3HdrOpts.set_secHdr(tempVar1);

      pdu.msgType = new _NAS_PROTOCOL_CLASS_msgType(_TS24301MsgsValues.mt_TrkAreaUpdRequest);
      pdu.data = pvalue;
      pvalue.nasKeySetIdentifier = new NASKeySetIdentifier();
      pvalue.nasKeySetIdentifier.tsc = new Asn1Boolean(false);
      pvalue.nasKeySetIdentifier.ident = new Asn1Integer(3);
      pvalue.epsUpdateType = new EPSUpdateType();
      pvalue.epsUpdateType.active = new Asn1Boolean(false);
      pvalue.epsUpdateType.value = new Asn1Integer(6);
      pvalue.oldGUTI = new EPSMobileIdentity();
      pvalue.oldGUTI.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.oldGUTI.typeOfIdent = EPSMobileIdentity_typeOfIdent.imsi();
      pvalue.oldGUTI.content = new EPSMobileIdentity_content();
      pvalue.oldGUTI.content.set_imsi(new IMSI("33805123456789"));
      pvalue.nonCurrNativeNASKeySetIdent = new NASKeySetIdentifier();
      pvalue.nonCurrNativeNASKeySetIdent.tsc = new Asn1Boolean(false);
      pvalue.nonCurrNativeNASKeySetIdent.ident = new Asn1Integer(5);
      pvalue.gprsCiphKeySeqNum = new CiphKeySeqNum();
      pvalue.gprsCiphKeySeqNum.spare = new Asn1Boolean(false);
      pvalue.gprsCiphKeySeqNum.value = new Asn1Integer(1);
      pvalue.oldP_TMSISignature = new P_TMSISignature();
      {
          byte testdata[] = {
         (byte)0x81, (byte)0x99, (byte)0xf8 };
         pvalue.oldP_TMSISignature.value = new Asn1OctetString(testdata);
      }
      pvalue.additionalGUTI = new EPSMobileIdentity();
      pvalue.additionalGUTI.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.additionalGUTI.filler = new Asn1Integer(14);
      pvalue.additionalGUTI.oddEvenInd = new Asn1Boolean(false);
      pvalue.additionalGUTI.typeOfIdent = EPSMobileIdentity_typeOfIdent.imsi();
      pvalue.additionalGUTI.content = new EPSMobileIdentity_content();
      pvalue.additionalGUTI.content.set_imsi(new IMSI("24702123456789"));
      {
          byte testdata[] = {
         (byte)0x82, (byte)0x61, (byte)0x3e, (byte)0xbd };
         pvalue.nonce = new Nonce();
         pvalue.nonce.value = testdata;
      }
      pvalue.ueNetworkCapability = new UENetworkCapability();
      pvalue.ueNetworkCapability.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.ueNetworkCapability.epsEncryptAlgs = new EPSEncryptAlgsSupported();
      pvalue.ueNetworkCapability.epsEncryptAlgs.eea0 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.epsEncryptAlgs.x128_eea1 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.epsEncryptAlgs.x128_eea2 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.epsEncryptAlgs.eea3 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.epsEncryptAlgs.eea4 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.epsEncryptAlgs.eea5 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.epsEncryptAlgs.eea6 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.epsEncryptAlgs.eea7 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.epsIntegAlgs = new EPSIntegAlgsSupported();
      pvalue.ueNetworkCapability.epsIntegAlgs.eia0 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.epsIntegAlgs.eia1 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.epsIntegAlgs.eia2 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.epsIntegAlgs.eia3 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.epsIntegAlgs.eia4 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.epsIntegAlgs.eia5 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.epsIntegAlgs.eia6 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.epsIntegAlgs.eia7 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.umtsEncryptAlgs = new UMTSEncryptAlgsSupported();
      pvalue.ueNetworkCapability.umtsEncryptAlgs.uea0 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.umtsEncryptAlgs.uea1 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.umtsEncryptAlgs.uea2 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.umtsEncryptAlgs.uea3 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.umtsEncryptAlgs.uea4 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.umtsEncryptAlgs.uea5 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.umtsEncryptAlgs.uea6 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.umtsEncryptAlgs.uea7 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.octet6 = new UENetworkCapability_octet6();
      pvalue.ueNetworkCapability.octet6.ucs2 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.octet6.umtsIntegAlgs = new UMTSIntegAlgsSupported();
      pvalue.ueNetworkCapability.octet6.umtsIntegAlgs.uia1 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.octet6.umtsIntegAlgs.uia2 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.octet6.umtsIntegAlgs.uia3 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.octet6.umtsIntegAlgs.uia4 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.octet6.umtsIntegAlgs.uia5 = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.octet6.umtsIntegAlgs.uia6 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.octet6.umtsIntegAlgs.uia7 = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.octet7 = new UENetworkCapability_octet7();
      pvalue.ueNetworkCapability.octet7.proSedd = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.octet7.proSe = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.octet7.h245_ash = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.octet7.acc_csfb = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.octet7.lpp = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.octet7.lcs = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.octet7.x1xSR_VCC = new Asn1Boolean(true);
      pvalue.ueNetworkCapability.octet7.nf = new Asn1Boolean(false);
      pvalue.ueNetworkCapability.spare4 = new Asn1OctetString();
      pvalue.ueNetworkCapability.spare4.value = new byte[0];
      pvalue.lastVisitedRegisteredTAI = new TrackingAreaIdent();
      pvalue.lastVisitedRegisteredTAI.mcc_mnc = new MCCMNCPair();
      pvalue.lastVisitedRegisteredTAI.mcc_mnc.mcc = new TBCDString3("525");
      pvalue.lastVisitedRegisteredTAI.mcc_mnc.mnc = new TBCDString3("03");
      {
          byte testdata[] = {
         (byte)0xb0, 0xb };
         pvalue.lastVisitedRegisteredTAI.tac = new TrackingAreaCode();
         pvalue.lastVisitedRegisteredTAI.tac.value = testdata;
      }
      pvalue.drxParameter = new DRXParameter();
      pvalue.drxParameter.splitPGCycleCode = new Asn1Integer(173);
      pvalue.drxParameter.lenCoefAndValue = new Asn1Integer(7);
      pvalue.drxParameter.splitOnCCCH = new Asn1Boolean(false);
      pvalue.drxParameter.nonDRXTimer = new Asn1Integer(6);
      pvalue.ueRadioCapInfoUpdateNeeded = new UERadioCapInfoUpdateNeeded();
      pvalue.ueRadioCapInfoUpdateNeeded.spare = new Asn1Integer(1);
      pvalue.ueRadioCapInfoUpdateNeeded.value = new Asn1Boolean(false);
      pvalue.epsBearerContextStatus = new EPSBearerContextStatus();
      pvalue.epsBearerContextStatus.ebi7 = new Asn1Boolean(true);
      pvalue.epsBearerContextStatus.ebi6 = new Asn1Boolean(false);
      pvalue.epsBearerContextStatus.ebi5 = new Asn1Boolean(true);
      pvalue.epsBearerContextStatus.ebi4 = new Asn1Boolean(false);
      pvalue.epsBearerContextStatus.ebi3 = new Asn1Boolean(true);
      pvalue.epsBearerContextStatus.ebi2 = new Asn1Boolean(false);
      pvalue.epsBearerContextStatus.ebi1 = new Asn1Boolean(true);
      pvalue.epsBearerContextStatus.ebi0 = new Asn1Boolean(true);
      pvalue.epsBearerContextStatus.ebi15 = new Asn1Boolean(true);
      pvalue.epsBearerContextStatus.ebi14 = new Asn1Boolean(false);
      pvalue.epsBearerContextStatus.ebi13 = new Asn1Boolean(false);
      pvalue.epsBearerContextStatus.ebi12 = new Asn1Boolean(false);
      pvalue.epsBearerContextStatus.ebi11 = new Asn1Boolean(true);
      pvalue.epsBearerContextStatus.ebi10 = new Asn1Boolean(false);
      pvalue.epsBearerContextStatus.ebi9 = new Asn1Boolean(true);
      pvalue.epsBearerContextStatus.ebi8 = new Asn1Boolean(true);
      pvalue.msNetworkCapability = new MSNetworkCapability();
      pvalue.msNetworkCapability.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.msNetworkCapability.octet1 = new MSNetworkCapability_octet1();
      pvalue.msNetworkCapability.octet1.gea1 = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet1.smCapDeicatedChan = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet1.smCapGPRSChan = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet1.ucs2Support = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet1.ssScreeningInd = new Asn1Integer(2);
      pvalue.msNetworkCapability.octet1.soLSACap = new Asn1Boolean(true);
      pvalue.msNetworkCapability.octet1.revisionLevelInd = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet2 = new MSNetworkCapability_octet2();
      pvalue.msNetworkCapability.octet2.pfcFeatureMode = new Asn1Boolean(true);
      pvalue.msNetworkCapability.octet2.extendedGEA = new Asn1Integer(60);
      pvalue.msNetworkCapability.octet2.lcsVACap = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet3 = new MSNetworkCapability_octet3();
      pvalue.msNetworkCapability.octet3.psInterRATHOToUTRANIuModeCap = new Asn1Boolean(true);
      pvalue.msNetworkCapability.octet3.psInterRATHOToEUTRANS1ModeCap = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet3.emmCombinedProcCap = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet3.isrSupport = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet3.srvccToGERAN_UTRANCap = new Asn1Boolean(true);
      pvalue.msNetworkCapability.octet3.epcCap = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet3.nfCapability = new Asn1Boolean(true);
      pvalue.msNetworkCapability.octet3.geranNetShareCap = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet4 = new MSNetworkCapability_octet4();
      pvalue.msNetworkCapability.octet4.userPlaneIntegProtSupp = new Asn1Boolean(true);
      pvalue.msNetworkCapability.octet4.gia_4 = new Asn1Boolean(true);
      pvalue.msNetworkCapability.octet4.gia_5 = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet4.gia_6 = new Asn1Boolean(true);
      pvalue.msNetworkCapability.octet4.gia_7 = new Asn1Boolean(true);
      pvalue.msNetworkCapability.octet4.epcoIeIndicator = new Asn1Boolean(false);
      pvalue.msNetworkCapability.octet4.restrictUseEnhancedCovCap = new Asn1Boolean(true);
      pvalue.msNetworkCapability.octet4.dualConnEutraNRCap = new Asn1Boolean(false);
      pvalue.msNetworkCapability.spareBits = new Asn1BitString();
      pvalue.msNetworkCapability.spareBits.numbits = 11;
      {
         byte[] bitData = new byte[2];
         bitData[0] = (byte) 0x4f;
         bitData[1] = (byte) 0x39;
         pvalue.msNetworkCapability.spareBits.value = bitData;
      }
      pvalue.oldLocAreaIdent = new LocAreaIdent();
      pvalue.oldLocAreaIdent.mcc_mnc = new MCCMNCPair();
      pvalue.oldLocAreaIdent.mcc_mnc.mcc = new TBCDString3("547");
      pvalue.oldLocAreaIdent.mcc_mnc.mnc = new TBCDString3("20");
      pvalue.oldLocAreaIdent.lac = new Asn1Integer(12004);
      pvalue.tmsiStatus = new TMSIStatus();
      pvalue.tmsiStatus.spare = new Asn1Integer(3);
      pvalue.tmsiStatus.flag = new Asn1Boolean(false);
      pvalue.mobileStationClassmark2 = new MobileStationClassmark2();
      pvalue.mobileStationClassmark2.cm1 = new MobileStationClassmark1();
      pvalue.mobileStationClassmark2.cm1.spare = new Asn1Boolean(false);
      pvalue.mobileStationClassmark2.cm1.revLevel = new Asn1Integer(0);
      pvalue.mobileStationClassmark2.cm1.esInd = new Asn1Boolean(false);
      pvalue.mobileStationClassmark2.cm1.a5_1_alg = new Asn1Boolean(true);
      pvalue.mobileStationClassmark2.cm1.rfPowerCap = RFPowerCapValues.class4();
      pvalue.mobileStationClassmark2.octet4 = new MobileStationClassmark2_octet4();
      pvalue.mobileStationClassmark2.octet4.spare = new Asn1Boolean(false);
      pvalue.mobileStationClassmark2.octet4.psCapability = new Asn1Boolean(false);
      pvalue.mobileStationClassmark2.octet4.ssScreeningInd = new Asn1Integer(2);
      pvalue.mobileStationClassmark2.octet4.smCapability = new Asn1Boolean(true);
      pvalue.mobileStationClassmark2.octet4.vbsNotifRecept = new Asn1Boolean(true);
      pvalue.mobileStationClassmark2.octet4.vgcsNotifRecept = new Asn1Boolean(false);
      pvalue.mobileStationClassmark2.octet4.fcFreqCap = new Asn1Boolean(false);
      pvalue.mobileStationClassmark2.octet5 = new MobileStationClassmark2_octet5();
      pvalue.mobileStationClassmark2.octet5.cm3 = new Asn1Boolean(true);
      pvalue.mobileStationClassmark2.octet5.spare = new Asn1Boolean(false);
      pvalue.mobileStationClassmark2.octet5.lcsvaCap = new Asn1Boolean(true);
      pvalue.mobileStationClassmark2.octet5.ucs2Treatment = new Asn1Boolean(false);
      pvalue.mobileStationClassmark2.octet5.soLSA = new Asn1Boolean(true);
      pvalue.mobileStationClassmark2.octet5.cmsp = new Asn1Boolean(true);
      pvalue.mobileStationClassmark2.octet5.a5_3_alg = new Asn1Boolean(true);
      pvalue.mobileStationClassmark2.octet5.a5_2_alg = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3 = new MobileStationClassmark3();
      pvalue.mobileStationClassmark3.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.mobileStationClassmark3.spare = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.multiband = new MultibandSupported();
      pvalue.mobileStationClassmark3.multiband.supported = MBSEnum.mbs0();
      pvalue.mobileStationClassmark3.multiband.a5bits = new A5Bits();
      pvalue.mobileStationClassmark3.multiband.a5bits.a5_7 = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.multiband.a5bits.a5_6 = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.multiband.a5bits.a5_5 = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.multiband.a5bits.a5_4 = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.rSupport = new RSupport(2);
      pvalue.mobileStationClassmark3.hscsdMultiSlotCap = new HSCSDMultiSlotCap(0);
      pvalue.mobileStationClassmark3.ucs2Treatment = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.extMeasurementCap = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.msMeasurementCap = new MSMeasurementCap();
      pvalue.mobileStationClassmark3.msMeasurementCap.smsValue = new Asn1Integer(4);
      pvalue.mobileStationClassmark3.msMeasurementCap.smValue = new Asn1Integer(14);
      pvalue.mobileStationClassmark3.msPositioningMethodCap = new Asn1Integer(29);
      pvalue.mobileStationClassmark3.ecsdMultiSlotCap = new Asn1Integer(21);
      pvalue.mobileStationClassmark3.an_8_PSKStruct = new Type8PSKStruct();
      pvalue.mobileStationClassmark3.an_8_PSKStruct.modulationCap = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.an_8_PSKStruct.rfPowerCap1 = new Asn1Integer(2);
      pvalue.mobileStationClassmark3.an_8_PSKStruct.rfPowerCap2 = new Asn1Integer(2);
      pvalue.mobileStationClassmark3.gsm400Struct = new GSM400Struct();
      pvalue.mobileStationClassmark3.gsm400Struct.bandsSupported = GSM400Struct_bandsSupported.band1();
      pvalue.mobileStationClassmark3.gsm400Struct.assocRadioCap = new Asn1Integer(13);
      pvalue.mobileStationClassmark3.gsm850AssocRadioCap = new Asn1Integer(1);
      pvalue.mobileStationClassmark3.gsm1900AssocRadioCap = new Asn1Integer(9);
      pvalue.mobileStationClassmark3.umtsFDDRadioAccessTechCap = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.umts384McpsTDDRadioAccessTechCap = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.cdma2000RadioAccessTechCap = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.dtmGPRSStruct = new DTMGPRSStruct();
      pvalue.mobileStationClassmark3.dtmGPRSStruct.dtmGPRSMultiSlotClass = new Asn1Integer(1);
      pvalue.mobileStationClassmark3.dtmGPRSStruct.singleSlotDTM = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.dtmGPRSStruct.dtmEGPRSMultiSlotClass = new Asn1Integer(2);
      pvalue.mobileStationClassmark3.singleBandSupport = new Asn1Integer(10);
      pvalue.mobileStationClassmark3.gsm750AssocRadioCap = new Asn1Integer(5);
      pvalue.mobileStationClassmark3.umts1_28McpsTDDRadioAccessTechCap = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.geranFeaturePackage1 = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.extDtmGPRSStruct = new MobileStationClassmark3_extDtmGPRSStruct();
      pvalue.mobileStationClassmark3.extDtmGPRSStruct.extDtmGPRSMultiSlotClass = new Asn1Integer(2);
      pvalue.mobileStationClassmark3.extDtmGPRSStruct.extDtmEGPRSMultiSlotClass = new Asn1Integer(1);
      pvalue.mobileStationClassmark3.highMultislotCap = new Asn1Integer(0);
      pvalue.mobileStationClassmark3.geranIuModeCap = new GERANIuModeCap();
      pvalue.mobileStationClassmark3.geranIuModeCap.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.mobileStationClassmark3.geranIuModeCap.floIuCap = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.geranIuModeCap.spareBits = new Asn1BitString();
      pvalue.mobileStationClassmark3.geranIuModeCap.spareBits.value = new byte[1];
      pvalue.mobileStationClassmark3.geranIuModeCap.spareBits.numbits = 3;
      pvalue.mobileStationClassmark3.geranIuModeCap.spareBits.value = new byte[1];
      pvalue.mobileStationClassmark3.geranIuModeCap.spareBits.value[0] = (byte)0x67;
      pvalue.mobileStationClassmark3.geranFeaturePackage2 = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.gmskMultislotPowerProfile = new Asn1Integer(1);
      pvalue.mobileStationClassmark3.multislotPowerProfile8PSK = new Asn1Integer(2);
      pvalue.mobileStationClassmark3.tgsm400Struct = new GSM400Struct();
      pvalue.mobileStationClassmark3.tgsm400Struct.bandsSupported = GSM400Struct_bandsSupported.band1();
      pvalue.mobileStationClassmark3.tgsm400Struct.assocRadioCap = new Asn1Integer(2);
      pvalue.mobileStationClassmark3.zerobit = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.downlinkAdvancedRcvrPerf = new Asn1Integer(0);
      pvalue.mobileStationClassmark3.dtmEnhancementsCap = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.dtmGPRSStruct2 = new MobileStationClassmark3_dtmGPRSStruct2();
      pvalue.mobileStationClassmark3.dtmGPRSStruct2.dtmGPRSHighMultiSlotClass = new Asn1Integer(6);
      pvalue.mobileStationClassmark3.dtmGPRSStruct2.offsetRequired = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.dtmGPRSStruct2.dtmEGPRSHighMultiSlotClass = new Asn1Integer(1);
      pvalue.mobileStationClassmark3.repeatedACCHCap = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.gsm710AssocRadioCap = new Asn1Integer(11);
      pvalue.mobileStationClassmark3.tgsm810AssocRadioCap = new Asn1Integer(14);
      pvalue.mobileStationClassmark3.cipheringModeSettingCap = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.additionalPositioningCap = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.e_UTRA_FDD_support = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.e_UTRA_TDD_support = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.e_UTRA_MeasAndReportSupport = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.priorityBasedReselectionSupport = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.uTRA_CSG_Cells_Reporting = new Asn1Boolean(false);
      pvalue.mobileStationClassmark3.vAMOS_Level = new Asn1Integer(2);
      pvalue.mobileStationClassmark3.tighterCapability = new Asn1Integer(0);
      pvalue.mobileStationClassmark3.selCiphOf_DL_SACCH = new Asn1Boolean(true);
      pvalue.mobileStationClassmark3.spareBits = new Asn1BitString();  
      pvalue.mobileStationClassmark3.spareBits.numbits = 2;
      {
         byte[] bitData = new byte[1];
         bitData[0] = (byte) 0x31;
         pvalue.mobileStationClassmark3.spareBits.value = bitData;
      }
      pvalue.supportedCodecs = new SupportedCodecList();
      pvalue.supportedCodecs.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.supportedCodecs.codecs = new SupportedCodecList_codecs();
      pvalue.supportedCodecs.codecs.elements = new SupportedCodec[1];
      pvalue.supportedCodecs.codecs.elements[0] = new SupportedCodec();
      pvalue.supportedCodecs.codecs.elements[0].sysID = new Asn1Integer(19);
      {
          byte testdata[] = {
         (byte)0xa0 };
         pvalue.supportedCodecs.codecs.elements[0].codecBitmap = new Asn1OctetString();
         pvalue.supportedCodecs.codecs.elements[0].codecBitmap.value = testdata;
      }
      pvalue.addnUpdateType = new AddnUpdateType();
      pvalue.addnUpdateType.pnb_CIoT = AddnUpdateType_pnb_CIoT.noAddInfo();
      pvalue.addnUpdateType.saf = new Asn1Boolean(false);
      pvalue.addnUpdateType.autv = new Asn1Boolean(false);

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
