import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
   
      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.AuthRequest data = new AuthRequest();
      com.objsys.nas.TS24501Msgs.AuthRequest pvalue = data;

      /* Populate data structure */

      pdu.protoDiscr = NAS5GProtoDiscr.mobMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_mm tempVar1 = new PDU_hdrData_mm();
      pdu.hdrData.set_mm(tempVar1);

      tempVar1.secHdrType = NAS5GSecHdrType.noSec();
      tempVar1.spare1 = new Asn1Integer(0);

      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.mt_AuthRequest5G);
      pdu.data = pvalue;
      pvalue.nasKeySetIdent = new NASKeySetIdentifier();
      pvalue.nasKeySetIdent.tsc = new Asn1Boolean(true);
      pvalue.nasKeySetIdent.ident = new Asn1Integer(5);
      {
         byte testdata[] = { (byte)0x21, 0x12 };
         pvalue.abba = new ABBA(testdata);
      }
      {
         byte testdata[] = {
         (byte)0x21, (byte)0xfe, (byte)0xa6, (byte)0x04, (byte)0xea, (byte)0x84, (byte)0x28, (byte)0x97,
         (byte)0xf3, (byte)0xf9, (byte)0x04, (byte)0x7f, (byte)0xa0, (byte)0x20, (byte)0x6c, (byte)0x15 };
         pvalue.authParamRAND = new AuthParamRAND();
         pvalue.authParamRAND.value = testdata;
      }
      {
         byte testdata[] = {
         (byte)0x18, (byte)0xcf, (byte)0x47, (byte)0x8c, (byte)0xe2, (byte)0xaf, (byte)0xc8, (byte)0xfd,
         (byte)0x0f, (byte)0xe4, (byte)0xbb, (byte)0x9c, (byte)0x42, (byte)0xd7, (byte)0xa3, (byte)0x66 };
         pvalue.authParamAUTN = new AuthParamAUTN();
         pvalue.authParamAUTN.autn = new Asn1OctetString(testdata);
      }

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
