import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.ControlPlaneServReq data = new ControlPlaneServReq();
      com.objsys.nas.TS24501Msgs.ControlPlaneServReq pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.mobMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_mm tempVar1 = new PDU_hdrData_mm();
      pdu.hdrData.set_mm(tempVar1);

      tempVar1.secHdrType = NAS5GSecHdrType.noSec();
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.mt_ControlPlaneServReq5G);
      pdu.data = pvalue;
      pvalue.nasKeySetIdent = new NASKeySetIdentifier();
      pvalue.nasKeySetIdent.tsc = new Asn1Boolean(false);
      pvalue.nasKeySetIdent.ident = new Asn1Integer(6);
      pvalue.ctrlPlaneServType = new ControlPlaneServiceType();
      pvalue.ctrlPlaneServType.value = new Asn1Integer(4);
      pvalue.pduSessionId = new PDUSessionId2(160);
      pvalue.releaseAssistIndication = new ReleaseAssistInd();
      pvalue.releaseAssistIndication.ddx = ReleaseAssistInd_ddx.ddxSingleDLTrans();

      pvalue.mV16ExtPresent = true;
      pvalue.nasMsgContainer = new NASMessageContainer2();
      pvalue.nasMsgContainer.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.nasMsgContainer.contents = new Asn1OctetString();
      {
          byte testdata[] = { 
         (byte)0xbe, (byte)0x97, (byte)0xf1, (byte)0x28 };
         pvalue.nasMsgContainer.contents.value = testdata;
      }

      pvalue.ciotSmallDataContainer = new CIoTSmallDataContainer();
      pvalue.ciotSmallDataContainer.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.ciotSmallDataContainer.dataType = CIoTDataTypes.ciotCtrlPlaneUserData();
      pvalue.ciotSmallDataContainer.containerContents =
                 new CIoTSmallDataContainer_containerContents();
      CIoTSmallDataContainer_containerContents_ctrlPlaneUserData userData =
               new CIoTSmallDataContainer_containerContents_ctrlPlaneUserData(2, 7);
      pvalue.ciotSmallDataContainer.containerContents.set_ctrlPlaneUserData(userData);
      pvalue.ciotSmallDataContainer.contents = new Asn1OctetString();
      {
         byte testdata[] = { (byte)0xfb, (byte)0xea, (byte)0xd9, (byte)0xc8 };
         pvalue.ciotSmallDataContainer.contents.value = testdata;
      }

      pvalue.mV17ExtPresent = true;
      pvalue.allowedPduSessStat = new AllowedPDUSessionStatus();
      pvalue.allowedPduSessStat.length = new Asn1Integer(0);  // Calculated automatically...
      pvalue.allowedPduSessStat.psi_7 = new Asn1Boolean(true);
      pvalue.allowedPduSessStat.psi_2 = new Asn1Boolean(true);
      pvalue.allowedPduSessStat.psi_13 = new Asn1Boolean(true);
      pvalue.allowedPduSessStat.psi_8 = new Asn1Boolean(true);
      pvalue.allowedPduSessStat.psi_0 = new Asn1Boolean(false);
      pvalue.allowedPduSessStat.psi_1 = new Asn1Boolean(false);
      pvalue.allowedPduSessStat.psi_3 = new Asn1Boolean(false);
      pvalue.allowedPduSessStat.psi_4 = new Asn1Boolean(false);
      pvalue.allowedPduSessStat.psi_5 = new Asn1Boolean(false);
      pvalue.allowedPduSessStat.psi_6 = new Asn1Boolean(false);
      pvalue.allowedPduSessStat.psi_9 = new Asn1Boolean(false);
      pvalue.allowedPduSessStat.psi_10 = new Asn1Boolean(false);
      pvalue.allowedPduSessStat.psi_11 = new Asn1Boolean(false);
      pvalue.allowedPduSessStat.psi_12 = new Asn1Boolean(false);
      pvalue.allowedPduSessStat.psi_14 = new Asn1Boolean(false);
      pvalue.allowedPduSessStat.psi_15 = new Asn1Boolean(false);

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
