import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded

      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.DeregReqUEOrig data = new DeregReqUEOrig();
      com.objsys.nas.TS24501Msgs.DeregReqUEOrig pvalue = data;
      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.mobMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_mm tempVar1 = new PDU_hdrData_mm();
      pdu.hdrData.set_mm(tempVar1);

      tempVar1.secHdrType = NAS5GSecHdrType.noSec();
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.mt_DeregReqUEOrig5G);
      pdu.data = pvalue;

      pvalue.nasKeySetIdent = new NASKeySetIdentifier();
      pvalue.nasKeySetIdent.tsc = new Asn1Boolean(true);
      pvalue.nasKeySetIdent.ident = new Asn1Integer(3);
      pvalue.deregType = new DeregistrationType();
      pvalue.deregType.switchOff = new Asn1Boolean(true);
      pvalue.deregType.reregReqd = new Asn1Boolean(false);
      pvalue.deregType.accessType = new Asn1Integer(1);
      pvalue.mobileId = new com.objsys.nas.TS24501IES.MobileIdentity();
      MobileIdentity_suci tempVar2 = new MobileIdentity_suci();
      pvalue.mobileId.set_suci(tempVar2);

      MobileIdentity_SUCI_IMSI tempVar3 = new MobileIdentity_SUCI_IMSI();
      tempVar2.set_supiImsi(tempVar3);

      tempVar3.mcc_mnc = new MCCMNCPair();
      tempVar3.mcc_mnc.mcc = new TBCDString3("789");
      tempVar3.mcc_mnc.mnc = new TBCDString3("12a");
      tempVar3.routingInd1 = new Asn1Integer(1);
      tempVar3.routingInd2 = new Asn1Integer(3);
      tempVar3.routingInd3 = new Asn1Integer(5);
      tempVar3.routingInd4 = new Asn1Integer(7);
      tempVar3.protSchemeId = new Asn1Integer(15);
      tempVar3.homeNetId = new Asn1Integer(63);
      tempVar3.schemeOutput = new Asn1OctetString();
      {
          byte testdata[] = { 
         (byte)0x71, (byte)0xad, (byte)0x0f, (byte)0xcf };
         tempVar3.schemeOutput.value = testdata;
      }

      // Create a message buffer object and encode the record
      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
