import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.DeregReqUETerm data = new DeregReqUETerm();
      com.objsys.nas.TS24501Msgs.DeregReqUETerm pvalue = data;
      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.mobMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_mm tempVar1 = new PDU_hdrData_mm();
      pdu.hdrData.set_mm(tempVar1);

      tempVar1.secHdrType = NAS5GSecHdrType.noSec();
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.mt_DeregReqUETerm5G);
      pdu.data = pvalue;
      pvalue.deregType = new DeregistrationType();
      pvalue.deregType.switchOff = new Asn1Boolean(false);
      pvalue.deregType.reregReqd = new Asn1Boolean(true);
      pvalue.deregType.accessType = new Asn1Integer(2);
      pvalue.cause = new MMCause(5);
      pvalue.t3346 = new GPRSTimer2();
      pvalue.t3346.value = new GPRSTimer();
      pvalue.t3346.value.unit = new Asn1Integer(5);
      pvalue.t3346.value.timerValue = new Asn1Integer(21);
      pvalue.mV16ExtPresent = true;
      pvalue.rejectedNSSAI = new RejectedNSSAIList();
      pvalue.rejectedNSSAI.rejectedNSSAI = new RejectedNSSAI[1];
      pvalue.rejectedNSSAI.rejectedNSSAI[0] = new RejectedNSSAI();
      pvalue.rejectedNSSAI.rejectedNSSAI[0].cause = RejectedNSSAI_cause.authPending();
      pvalue.rejectedNSSAI.rejectedNSSAI[0].sst = new SST(150);
      pvalue.rejectedNSSAI.rejectedNSSAI[0].sd = new SD(65000);

      pvalue.mV17ExtPresent = true;
      pvalue.extCAGInfoList = new ExtCAGInfoList();
      pvalue.extCAGInfoList.length = new Asn1Integer(0); // Calculated automatically...
      pvalue.extCAGInfoList.entries = new ExtCAGEntry[1];
      pvalue.extCAGInfoList.entries[0] = new ExtCAGEntry();
      pvalue.extCAGInfoList.entries[0].length = new Asn1Integer(0); // Calculated automatically...
      pvalue.extCAGInfoList.entries[0].mcc_mnc = new MCCMNCPair();
      pvalue.extCAGInfoList.entries[0].mcc_mnc.mcc = new TBCDString3("123");
      pvalue.extCAGInfoList.entries[0].mcc_mnc.mnc = new TBCDString3("456");
      pvalue.extCAGInfoList.entries[0].caili = new Asn1Integer(1);
      pvalue.extCAGInfoList.entries[0].lci = new Asn1Boolean(false);
      pvalue.extCAGInfoList.entries[0].cagOnly = new Asn1Boolean(true);
      pvalue.extCAGInfoList.entries[0].cagIdIndLen = new Asn1Integer(4);
      {
         byte testdata[] = { (byte)0x01, (byte)0x04, (byte)0x01, (byte)0x04 };
         pvalue.extCAGInfoList.entries[0].cagId = new Asn1OctetString(testdata);
      }
      pvalue.extCAGInfoList.entries[0].cagIdAddlInfoList = new ExtCAGEntry_cagIdAddlInfoList();
      CAGIdAddlInfoList tempVar2 = new CAGIdAddlInfoList();
      tempVar2.length = new Asn1Integer(0); // Calculated automatically...
      tempVar2.cagIdAddlInfo = new CAGIdAddlInfo[1];
      tempVar2.cagIdAddlInfo[0] = new CAGIdAddlInfo();
      tempVar2.cagIdAddlInfo[0].length = new Asn1Integer(0); // Calculated automatically...
      {
         byte testdata[] = { (byte)0x04, (byte)0x06, (byte)0x08, (byte)0x04 };
         tempVar2.cagIdAddlInfo[0].cagId = new Asn1OctetString(testdata);
      }
      tempVar2.cagIdAddlInfo[0].tvii = new Asn1Integer(0);
      tempVar2.cagIdAddlInfo[0].timeValidityInfo = new CAGIdAddlInfo_timeValidityInfo();
      tempVar2.cagIdAddlInfo[0].timeValidityInfo.set_absent();
      pvalue.extCAGInfoList.entries[0].cagIdAddlInfoList.set_present(tempVar2);

      // Create a message buffer object and encode the record
      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing
      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
