import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.EstablishReject data = new EstablishReject();
      com.objsys.nas.TS24501Msgs.EstablishReject pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.sessMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_sm tempVar1 = new PDU_hdrData_sm();
      pdu.hdrData.set_sm(tempVar1);

      tempVar1.pduSessIdent = PDUSessionIdentity.pduId1();
      tempVar1.procTransIdent = new Asn1Integer(120);
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.st_EstablishReject5G);
      pdu.data = pvalue;
      pvalue.cause = new SMCause(31);
      pvalue.backoffTimer = new GPRSTimer3();
      pvalue.backoffTimer.unit = GPRSTimer3_unit.valIncrMult10Mins();
      pvalue.backoffTimer.timerValue = new Asn1Integer(5);
      pvalue.allowedSSCMode = new AllowedSSCMode();
      pvalue.allowedSSCMode.ssc3 = new Asn1Boolean(true);
      pvalue.allowedSSCMode.ssc2 = new Asn1Boolean(false);
      pvalue.allowedSSCMode.ssc1 = new Asn1Boolean(true);
      pvalue.eapMessage = new EAPMessage();
      pvalue.eapMessage.length = new Asn1Integer(0); // Calculated automatically...
      pvalue.eapMessage.packetFormat = new EAPPacketFormat();
      pvalue.eapMessage.packetFormat.code = EAPFormatCode.request();
      pvalue.eapMessage.packetFormat.id = new Asn1Integer(62);
      pvalue.eapMessage.packetFormat.length = new Asn1Integer(8);
      pvalue.eapMessage.packetFormat.data = new Asn1OctetString();
      {
          byte tmpdata[] = { (byte)0xad, (byte)0xfa, (byte)0x79, (byte)0xe5 };
         pvalue.eapMessage.packetFormat.data.value = tmpdata;
      }
      pvalue.extProtoCfgOpts = new ExtProtoConfigOptions();
      pvalue.extProtoCfgOpts.octet4 = new ExtProtoConfigOptions_octet4();
      pvalue.extProtoCfgOpts.octet4.configProtocol = new Asn1Integer(0);

      pvalue.mV16ExtPresent = true;
      pvalue.reattemptInd = new ReattemptIndicator();
      pvalue.reattemptInd.eplmnc = new Asn1Boolean(true);
      pvalue.reattemptInd.ratc = new Asn1Boolean(false);

      pvalue.mV17ExtPresent = true;
      pvalue.aaContainer = new SLAAContainer();
      pvalue.aaContainer.length = new Asn1Integer(0); // Calculated automatically...
      pvalue.aaContainer.contents = new SLAAParameter[1];
      pvalue.aaContainer.contents[0] = new SLAAParameter();
      pvalue.aaContainer.contents[0].type = SLAAParamTypes.ptPendingInd();
      pvalue.aaContainer.contents[0].param = new SLAAParameter_param();
      SLAAPendingIndication tempVar2 = new SLAAPendingIndication();
      pvalue.aaContainer.contents[0].param.set_pendingInd(tempVar2);
      tempVar2.slapi = new Asn1Boolean(true);

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
