import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.ModifyRequest data = new ModifyRequest();
      com.objsys.nas.TS24501Msgs.ModifyRequest pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.sessMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_sm tempVar1 = new PDU_hdrData_sm();
      pdu.hdrData.set_sm(tempVar1);

      tempVar1.pduSessIdent = PDUSessionIdentity.pduId6();
      tempVar1.procTransIdent = new Asn1Integer(125);
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.st_ModifyRequest5G);
      pdu.data = pvalue;

      pvalue.capability = new SMCapability();
      pvalue.capability.tpmic = new Asn1Boolean(false);
      pvalue.capability.atsll = new Asn1Integer(3);
      pvalue.capability.epts1 = new Asn1Boolean(true);
      pvalue.capability.mh6Pdu = new Asn1Boolean(false);
      pvalue.capability.rqos = new Asn1Boolean(true);
      pvalue.maxPktFilters = new MaxSupportedPacketFilters();
      pvalue.maxPktFilters.maxFilters = new Asn1Integer(1244);

      pvalue.extProtoCfgOpts = new ExtProtoConfigOptions();
      pvalue.extProtoCfgOpts.octet4 = new ExtProtoConfigOptions_octet4();
      pvalue.extProtoCfgOpts.octet4.configProtocol = new Asn1Integer(0);

      pvalue.mV16ExtPresent = true;
      pvalue.ethernetHdrCompressionCfg = new EthHdrCompressConfig();
      pvalue.ethernetHdrCompressionCfg.cidLength = new Asn1Integer(2);

      pvalue.mV17ExtPresent = true;
      pvalue.reqMBSContainer = new RequestedMBSContainer();
      pvalue.reqMBSContainer.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.reqMBSContainer.sessionInfo = new RequestedMBSContainer_sessionInfo();
      pvalue.reqMBSContainer.sessionInfo.elements = new MBSSessionInfo[1];
      pvalue.reqMBSContainer.sessionInfo.elements[0] = new MBSSessionInfo();
      pvalue.reqMBSContainer.sessionInfo.elements[0].operation = MBSOperations.join();
      pvalue.reqMBSContainer.sessionInfo.elements[0].type = MBSSessionIdTypes.ipv4();
      pvalue.reqMBSContainer.sessionInfo.elements[0].id = new MBSSessionInfo_id();

      MBSSessionInfo_id_ipv4 tempVar2 = new MBSSessionInfo_id_ipv4();
      pvalue.reqMBSContainer.sessionInfo.elements[0].id.set_ipv4(tempVar2);
      {
         byte testdata[] = { (byte)0xa1, (byte)0xb2, (byte)0xc3, (byte)0xd4 };
         tempVar2.srcAddrInfo = new Asn1OctetString(testdata);
      }                         
      {
         byte testdata[] = { (byte)0xe5, (byte)0xf6, (byte)0x78, (byte)0x90 };
         tempVar2.destAddrInfo = new Asn1OctetString(testdata);
      }                         

      pvalue.mV18ExtPresent = true;
      pvalue.delayBudget = new Non3GDelayBudget();
      pvalue.delayBudget.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.delayBudget.delayBudgetList = new DelayBudget[1];
      pvalue.delayBudget.delayBudgetList[0] = new DelayBudget();
      pvalue.delayBudget.delayBudgetList[0].value = new Asn1Integer(2023);
      pvalue.delayBudget.delayBudgetList[0].qfipi = new Asn1Integer(1);
      pvalue.delayBudget.delayBudgetList[0].pfpi = new Asn1Integer(0);
      pvalue.delayBudget.delayBudgetList[0].qfiInfo = new DelayBudget_qfiInfo();
      DelayBudget_qfiInfo_present tempVar3 = new DelayBudget_qfiInfo_present();
      tempVar3.numQFIs = new Asn1Integer(1);
      tempVar3.qfiList = new DelayBudget_qfiInfo_present_qfiList();
      tempVar3.qfiList.elements = new Asn1OctetString[1];
      {
         byte testdata[] = { (byte)0x3c };
         tempVar3.qfiList.elements[0] = new Asn1OctetString(testdata);
      }
      pvalue.delayBudget.delayBudgetList[0].qfiInfo.set_present(tempVar3);
      pvalue.delayBudget.delayBudgetList[0].pktFilterList = new DelayBudget_pktFilterList();
      pvalue.delayBudget.delayBudgetList[0].pktFilterList.set_absent();

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
