import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.RegComplete data = new RegComplete();
      com.objsys.nas.TS24501Msgs.RegComplete pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.mobMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_mm tempVar1 = new PDU_hdrData_mm();
      pdu.hdrData.set_mm(tempVar1);

      tempVar1.secHdrType = NAS5GSecHdrType.noSec();
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.mt_RegComplete5G);
      pdu.data = pvalue;

      pvalue.transparentContainer = new SORTransparentContainer();
      pvalue.transparentContainer.length = new Asn1Integer(0); // Calculated automatically...
      pvalue.transparentContainer.steeringOrAck = new SORTransparentContainer_steeringOrAck();
      SORSteering steering = new SORSteering();
      steering.ap = new Asn1Integer(1);
      steering.ack = new Asn1Boolean(true);
      steering.listType = new Asn1Integer(1);
      steering.listInd = new Asn1Boolean(true);
      steering.dataType = new Asn1Integer(0);
      {
          byte testdata[] = {
             (byte)0x69, (byte)0xe2, (byte)0x7e, (byte)0x01, (byte)0x02, (byte)0x03, (byte)0x04, (byte)0x05, (byte)0x06, (byte)0x07, (byte)0x08, (byte)0x09, (byte)0x0a, (byte)0x0b, (byte)0x0c, (byte)0x0d };
         steering.sorMAC = new Asn1OctetString(testdata);
      }
      {
          byte testdata[] = {
             (byte)0xba, (byte)0xde };
         steering.counter = new Asn1OctetString(testdata);
      }
      steering.content = new SORSteering_content();
      SORSteering_content_plmnInfo plmnInfo = new SORSteering_content_plmnInfo();
      PLMNTechList plmnList = new PLMNTechList();
      plmnList.length = new Asn1Integer(10);
      plmnList.plmnIds = new PLMNTechIDs[2];
      plmnList.plmnIds[0] = new PLMNTechIDs();
      plmnList.plmnIds[0].id = new Asn1OctetString();
      plmnList.plmnIds[0].id.value = new byte[3];
      plmnList.plmnIds[0].id.value[0] = (byte)0xaa;
      plmnList.plmnIds[0].id.value[1] = (byte)0xbb;
      plmnList.plmnIds[0].id.value[2] = (byte)0xcc;
      plmnList.plmnIds[0].techId = new Asn1OctetString();
      plmnList.plmnIds[0].techId.value = new byte[2];
      plmnList.plmnIds[0].techId.value[0] = (byte)0xdd;
      plmnList.plmnIds[0].techId.value[1] = (byte)0xee;
      plmnList.plmnIds[1] = new PLMNTechIDs();
      plmnList.plmnIds[1].id = new Asn1OctetString();
      plmnList.plmnIds[1].id.value = new byte[3];
      plmnList.plmnIds[1].id.value[0] = (byte)0xaa;
      plmnList.plmnIds[1].id.value[1] = (byte)0xbb;
      plmnList.plmnIds[1].id.value[2] = (byte)0xcc;
      plmnList.plmnIds[1].techId = new Asn1OctetString();
      plmnList.plmnIds[1].techId.value = new byte[2];
      plmnList.plmnIds[1].techId.value[0] = (byte)0xdd;
      plmnList.plmnIds[1].techId.value[1] = (byte)0xee;
      plmnList.addlParams = new AdditionalParams();
      plmnList.addlParams.sssli = new Asn1Integer(1);
      plmnList.addlParams.sssi = new Asn1Integer(1);
      plmnList.addlParams.sscmi = new Asn1Boolean(true);
      plmnList.addlParams.si = new Asn1Integer(1);
      plmnList.addlParams.sorCmci = new AdditionalParams_sorCmci();
      CMCIList cmciList = new CMCIList();
      cmciList.length = new Asn1Integer(0); // Calculated automatically...
      cmciList.cmciRules = new CMCIRules[1];
      cmciList.cmciRules[0] = new CMCIRules();
      cmciList.cmciRules[0].length = new Asn1Integer(0); // Calculated automatically...
      cmciList.cmciRules[0].tsorCmTimer = new GPRSTimer();
      cmciList.cmciRules[0].tsorCmTimer.unit = new Asn1Integer(4);
      cmciList.cmciRules[0].tsorCmTimer.timerValue = new Asn1Integer(22);
      cmciList.cmciRules[0].type = new Asn1Integer(3);
      cmciList.cmciRules[0].value = new CMCIRules_value();
      CMCIRules_value_sstSd sstSd = new CMCIRules_value_sstSd();
      sstSd.sst = new SST(250);
      sstSd.sd = new SD(20000);
      cmciList.cmciRules[0].value.set_sstSd(sstSd);
      plmnList.addlParams.sorCmci.set_cmcis(cmciList);
      plmnList.addlParams.sorSnpnSi = new AdditionalParams_sorSnpnSi();
      SNPNSI snpnsi = new SNPNSI();
      snpnsi.length = new Asn1Integer(0); // Calculated automatically...
      snpnsi.clgi = new Asn1Integer(0);
      snpnsi.clsi = new Asn1Integer(0);
      snpnsi.snpnList = new SNPNSI_snpnList();
      snpnsi.snpnList.set_absent();
      snpnsi.ginList = new SNPNSI_ginList();
      snpnsi.ginList.set_absent();
      plmnList.addlParams.sorSnpnSi.set_snpnsi(snpnsi);
      plmnList.addlParams.sorSnpnSiLs = new AdditionalParams_sorSnpnSiLs();
      SNPNSILS snpnsils = new SNPNSILS();
      snpnsils.length = new Asn1Integer(0); // Calculated automatically...
      snpnsils.clglsi = new Asn1Integer(0);
      snpnsils.clslsi = new Asn1Integer(1);
      snpnsils.localSnpnList = new SNPNSILS_localSnpnList();
      LocalizedSNPNList snpnList = new LocalizedSNPNList();
      snpnList.length = new Asn1Integer(0); // Calculated automatically...
      snpnList.snpnInfo = new LocalizedSNPNInfo[1];
      snpnList.snpnInfo[0] = new LocalizedSNPNInfo();
      snpnList.snpnInfo[0].length = new Asn1Integer(0); // Calculated automatically...
      snpnList.snpnInfo[0].lvii = new Asn1Integer(0);
      snpnList.snpnInfo[0].laii = new Asn1Integer(1);
      snpnList.snpnInfo[0].timeInd = new Asn1Integer(0);
      snpnList.snpnInfo[0].snpnId = new SNPNIdentity();
      snpnList.snpnInfo[0].snpnId.mcc_mnc = new MCCMNCPair();
      snpnList.snpnInfo[0].snpnId.mcc_mnc.mcc = new TBCDString3("620");
      snpnList.snpnInfo[0].snpnId.mcc_mnc.mnc = new TBCDString3("900");
      snpnList.snpnInfo[0].snpnId.assignMode = new Asn1Integer(15);
      snpnList.snpnInfo[0].snpnId.nidDigit2 = new Asn1Integer(0);
      snpnList.snpnInfo[0].snpnId.nidDigit1 = new Asn1Integer(1);
      snpnList.snpnInfo[0].snpnId.nidDigit4 = new Asn1Integer(2);
      snpnList.snpnInfo[0].snpnId.nidDigit3 = new Asn1Integer(3);
      snpnList.snpnInfo[0].snpnId.nidDigit6 = new Asn1Integer(4);
      snpnList.snpnInfo[0].snpnId.nidDigit5 = new Asn1Integer(5);
      snpnList.snpnInfo[0].snpnId.nidDigit8 = new Asn1Integer(6);
      snpnList.snpnInfo[0].snpnId.nidDigit7 = new Asn1Integer(7);
      snpnList.snpnInfo[0].snpnId.nidDigit10 = new Asn1Integer(8);
      snpnList.snpnInfo[0].snpnId.nidDigit9 = new Asn1Integer(9);
      snpnList.snpnInfo[0].timeValidity = new LocalizedSNPNInfo_timeValidity();
      snpnList.snpnInfo[0].timeValidity.set_absent();
      snpnList.snpnInfo[0].locationValidity = new LocalizedSNPNInfo_locationValidity();
      snpnList.snpnInfo[0].locationValidity.set_absent();
      snpnList.snpnInfo[0].locationAssistance = new LocalizedSNPNInfo_locationAssistance();
      LocationAssistanceInfo locAsst = new LocationAssistanceInfo();
      locAsst.taisi = new Asn1Integer(0);
      locAsst.taipi = new Asn1Integer(0);
      locAsst.gadi = new Asn1Integer(1);
      locAsst.geoAreaDescs = new LocationAssistanceInfo_geoAreaDescs();
      GeoAreaDescription geoAreaDesc = new GeoAreaDescription();
      geoAreaDesc.numDescriptions = new Asn1Integer(1);
      geoAreaDesc.descriptions = new AreaDescriptions[1];
      geoAreaDesc.descriptions[0] = new AreaDescriptions();
      geoAreaDesc.descriptions[0].shapeType = new Asn1Integer(6);
      {
         byte testdata[] = { (byte)0x69, (byte)0xe2, (byte)0x7e, (byte)0x01, (byte)0x02, (byte)0x03, (byte)0x04 };
         geoAreaDesc.descriptions[0].shapeDesc = new Asn1OctetString(testdata);
      }
      locAsst.geoAreaDescs.set_present(geoAreaDesc);
      snpnList.snpnInfo[0].locationAssistance.set_info(locAsst);
      locAsst.plmnTAI = new LocationAssistanceInfo_plmnTAI();
      locAsst.plmnTAI.set_absent();
      locAsst.snpnTAI = new LocationAssistanceInfo_snpnTAI();
      locAsst.snpnTAI.set_absent();
      snpnsils.localSnpnList.set_localSnpns(snpnList);
      plmnList.addlParams.sorSnpnSiLs.set_snpnsils(snpnsils);
      snpnsils.localGinList = new SNPNSILS_localGinList();
      snpnsils.localGinList.set_absent();
      plmnInfo.set_plmnList(plmnList);
      steering.content.set_plmnInfo(plmnInfo);
      pvalue.transparentContainer.steeringOrAck.set_steering(steering);

      // Create a message buffer object and encode the record
      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing
      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
