import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.RegReject data = new RegReject();
      com.objsys.nas.TS24501Msgs.RegReject pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.mobMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_mm tempVar1 = new PDU_hdrData_mm();
      pdu.hdrData.set_mm(tempVar1);

      tempVar1.secHdrType = NAS5GSecHdrType.noSec();
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.mt_RegReject5G);
      pdu.data = pvalue;
   
      pvalue.cause = new MMCause(3);
      pvalue.t3346 = new GPRSTimer2();
      pvalue.t3346.value = new GPRSTimer();
      pvalue.t3346.value.unit = new Asn1Integer(1);
      pvalue.t3346.value.timerValue = new Asn1Integer(10);
      pvalue.t3502 = new GPRSTimer2();
      pvalue.t3502.value = new GPRSTimer();
      pvalue.t3502.value.unit = new Asn1Integer(2);
      pvalue.t3502.value.timerValue = new Asn1Integer(13);
      pvalue.eapMessage = new EAPMessage();
      pvalue.eapMessage.length = new Asn1Integer(0); // Calculated automatically...
      pvalue.eapMessage.packetFormat = new EAPPacketFormat();
      pvalue.eapMessage.packetFormat.code = EAPFormatCode.failure();
      pvalue.eapMessage.packetFormat.id = new Asn1Integer(5);
      pvalue.eapMessage.packetFormat.length = new Asn1Integer(8);
      pvalue.eapMessage.packetFormat.data = new Asn1OctetString();
      pvalue.eapMessage.packetFormat.data.value = new byte[4];
      pvalue.eapMessage.packetFormat.data.value[0] = (byte)0x70;
      pvalue.eapMessage.packetFormat.data.value[1] = (byte)0xc6;
      pvalue.eapMessage.packetFormat.data.value[2] = (byte)0x63;
      pvalue.eapMessage.packetFormat.data.value[3] = (byte)0x38;

      pvalue.mV17ExtPresent = true;
      pvalue.extRejectedNSSAI = new ExtRejectedNSSAIList();
      pvalue.extRejectedNSSAI.length = new Asn1Integer(0); // Calculated automatically...

      pvalue.extRejectedNSSAI.rejectedNSSAI = new PartialExtRejectedNSSAI[1];
      pvalue.extRejectedNSSAI.rejectedNSSAI[0] = new PartialExtRejectedNSSAI();
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].listType = com.objsys.nas.TS24501IES.PERNTypes.pernWithoutTimer();
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].numElems = new Asn1Integer(1);
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].backoffTimer = new PartialExtRejectedNSSAI_backoffTimer();

      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI = new ExtRejectedNSSAI[1];
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0] = new ExtRejectedNSSAI();
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].length = new Asn1Integer(8);
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].cause = ExtRejectedNSSAI_cause.unavailInCurrPlmn();
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].sst = new SST(155);
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].sd = new SD(56000);
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].mappedSST = new Asn1OctetString();
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].mappedSST.value = new byte[1];
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].mappedSST.value[0] = (byte)0xab;
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].mappedSD = new Asn1OctetString();
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].mappedSD.value = new byte[3];
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].mappedSD.value[0] = (byte)0xcd;
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].mappedSD.value[1] = (byte)0xef;
      pvalue.extRejectedNSSAI.rejectedNSSAI[0].rejectedSNSSAI[0].mappedSD.value[2] = (byte)0x01;

      pvalue.disasterReturnRange = new RegistrationWaitRange();
      pvalue.disasterReturnRange.length = new Asn1Integer(0); // Calculated automatically...
      pvalue.disasterReturnRange.minWaitTime = new GPRSTimer();
      pvalue.disasterReturnRange.minWaitTime.unit = new Asn1Integer(0);
      pvalue.disasterReturnRange.minWaitTime.timerValue = new Asn1Integer(11);
      pvalue.disasterReturnRange.maxWaitTime = new GPRSTimer();
      pvalue.disasterReturnRange.maxWaitTime.unit = new Asn1Integer(7);
      pvalue.disasterReturnRange.maxWaitTime.timerValue = new Asn1Integer(22);

      pvalue.mV18ExtPresent = true;
      pvalue.n3iwfAddr = new N3IWFAddress();
      pvalue.n3iwfAddr.type = N3IWFAddressTypes.fqdn();
      pvalue.n3iwfAddr.entry = new N3IWFAddress_entry();
      pvalue.n3iwfAddr.entry.set_entryFQDN(new N3IWFAddress_entry_entryFQDN(15, "www.obj-sys.com"));
      pvalue.tnanInfo = new TNANInfo();
      pvalue.tnanInfo.length = new Asn1Integer(0); // Calculated automatically...
      pvalue.tnanInfo.ssidInd = new Asn1Integer(1);
      pvalue.tnanInfo.tngfInd = new Asn1Integer(0);
      pvalue.tnanInfo.tngf = new TNANInfo_tngf();
      pvalue.tnanInfo.tngf.set_absent();
      pvalue.tnanInfo.ss = new TNANInfo_ss();
      TNANInfo_ss_present tempVar2 = new TNANInfo_ss_present();
      tempVar2.ssidLen = new Asn1Integer(2);
      tempVar2.ssid = new Asn1OctetString();
      tempVar2.ssid.value = new byte[2];
      tempVar2.ssid.value[0] = (byte)0xff;
      tempVar2.ssid.value[1] = (byte)0xcc;
      pvalue.tnanInfo.ss.set_present(tempVar2);

      // Create a message buffer object and encode the record
      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing
      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
