import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.RelayKeyRequest data = new RelayKeyRequest();
      com.objsys.nas.TS24501Msgs.RelayKeyRequest pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.mobMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_mm tempVar1 = new PDU_hdrData_mm();
      pdu.hdrData.set_mm(tempVar1);

      tempVar1.secHdrType = NAS5GSecHdrType.noSec();
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.mt_RelayKeyRequest5G);
      pdu.data = pvalue;

      pvalue.prti = new ProSeRelayTransactionId(30);

      pvalue.keyRequestParams = new RelayKeyRequestParams();
      pvalue.keyRequestParams.length = new Asn1Integer(0); // Calculated automatically...
      {
         byte tmpdata[] = { (byte)0x88, (byte)0x99, (byte)0xaa };
         pvalue.keyRequestParams.serviceCode = new Asn1OctetString(tmpdata);
      }
      {
         byte testdata[] = new byte[16];
         for (int j = 0; j < 16; j++)
         {
            testdata[j] = (byte)(j + 1);
         }
         pvalue.keyRequestParams.nonce = new Asn1OctetString(testdata);
      }
      pvalue.keyRequestParams.ueIdentity = new RemoteUEIdentity();
      pvalue.keyRequestParams.ueIdentity.ruit = new Asn1Integer(1);
      pvalue.keyRequestParams.ueIdentity.ueid = new RemoteUEIdentity_ueid();
      {
         byte testdata[] = new byte[32];
         for (int j = 0; j < 32; j++)
         {
            testdata[j] = (byte)j;
         }
         Asn1OctetString tempVar2 = new Asn1OctetString(testdata);
         pvalue.keyRequestParams.ueIdentity.ueid.set_prukId(tempVar2);
      }

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
