import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.RemoteUEReport data = new RemoteUEReport();
      com.objsys.nas.TS24501Msgs.RemoteUEReport pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.sessMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_sm tempVar1 = new PDU_hdrData_sm();
      pdu.hdrData.set_sm(tempVar1);

      tempVar1.pduSessIdent = PDUSessionIdentity.pduId10();
      tempVar1.procTransIdent = new Asn1Integer(120);
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.st_RemUEReport5G);
      pdu.data = pvalue;

      pvalue.connectedCtxt = new RemoteUEContextList();
      pvalue.connectedCtxt.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.connectedCtxt.numUECtxts = new Asn1Integer(1);
      pvalue.connectedCtxt.contexts = new RemoteUEContext[1];
      pvalue.connectedCtxt.contexts[0] = new RemoteUEContext();
      pvalue.connectedCtxt.contexts[0].contextLength = new Asn1Integer(0); /* length will be calculated */
      pvalue.connectedCtxt.contexts[0].spare1 = new Asn1Integer(0);
      pvalue.connectedCtxt.contexts[0].ueidFormat = new Asn1Boolean(true);
      pvalue.connectedCtxt.contexts[0].ueidType = UEIDTypes.cpPrukId();
      pvalue.connectedCtxt.contexts[0].ueidLength = new Asn1Integer(8);
      pvalue.connectedCtxt.contexts[0].ueid = new Asn1OctetString();
      {
         byte tmpdata[] = { (byte)0xfe, (byte)0xed, (byte)0x00, (byte)0xff, (byte)0xcd, (byte)0xdc, (byte)0x12, (byte)0x34 };
         pvalue.connectedCtxt.contexts[0].ueid.value = tmpdata;
      }
      pvalue.connectedCtxt.contexts[0].spare2 = new Asn1Integer(0);
      pvalue.connectedCtxt.contexts[0].upri4i = new Asn1Boolean(true);
      pvalue.connectedCtxt.contexts[0].tpri4i = new Asn1Boolean(false);
      pvalue.connectedCtxt.contexts[0].protocol = RemoteUEProtocols.ipv4();
      pvalue.connectedCtxt.contexts[0].addrInfo = new RemoteUEContext_addrInfo();
      Asn1OctetString data2 = new Asn1OctetString();
      {
         byte tmpdata[] = { (byte)0xfe, (byte)0xed, (byte)0x00, (byte)0xff };
         data2.value = tmpdata;
      }
      pvalue.connectedCtxt.contexts[0].addrInfo.set_ipv4(data2);
      pvalue.connectedCtxt.contexts[0].hplmnId = new RemoteUEContext_hplmnId();
      pvalue.connectedCtxt.contexts[0].hplmnId.set_absent();

      // Create a message buffer object and encode the record
      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing
      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {
          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
