import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.SecModeCmd data = new SecModeCmd();
      com.objsys.nas.TS24501Msgs.SecModeCmd pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.mobMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_mm tempVar1 = new PDU_hdrData_mm();
      pdu.hdrData.set_mm(tempVar1);

      tempVar1.secHdrType = NAS5GSecHdrType.noSec();
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.mt_SecModeCmd5G);
      pdu.data = pvalue;
      pvalue.nasSecAlgorithm = new NASSecurityAlgorithm();
      pvalue.nasSecAlgorithm.protectionType = ProtectionTypes.ea1();
      pvalue.nasSecAlgorithm.cipherType = CipherTypes.ia7();
      pvalue.nasKeySetIdent = new NASKeySetIdentifier();
      pvalue.nasKeySetIdent.tsc = new Asn1Boolean(true);
      pvalue.nasKeySetIdent.ident = new Asn1Integer(5);
      pvalue.replayedUECapabilities = new com.objsys.nas.TS24501IES.UESecurityCapability();
      pvalue.replayedUECapabilities.length = new Asn1Integer(0); /* length will be calculated */
      pvalue.replayedUECapabilities.ea0 = new Asn1Boolean(true);
      pvalue.replayedUECapabilities.ea1 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.ea2 = new Asn1Boolean(true);
      pvalue.replayedUECapabilities.ea3 = new Asn1Boolean(true);
      pvalue.replayedUECapabilities.ea4 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.ea5 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.ea6 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.ea7 = new Asn1Boolean(false);      
      pvalue.replayedUECapabilities.ia0 = new Asn1Boolean(false);      
      pvalue.replayedUECapabilities.ia1 = new Asn1Boolean(false);      
      pvalue.replayedUECapabilities.ia2 = new Asn1Boolean(false);      
      pvalue.replayedUECapabilities.ia3 = new Asn1Boolean(false);      
      pvalue.replayedUECapabilities.ia4 = new Asn1Boolean(true);
      pvalue.replayedUECapabilities.ia5 = new Asn1Boolean(true);
      pvalue.replayedUECapabilities.ia6 = new Asn1Boolean(true);
      pvalue.replayedUECapabilities.ia7 = new Asn1Boolean(false);            
      pvalue.replayedUECapabilities.octet5 = new UESecurityCapability_octet5();
      pvalue.replayedUECapabilities.octet5.eea0 = new Asn1Boolean(true);
      pvalue.replayedUECapabilities.octet5.eea1 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet5.eea2 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet5.eea3 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet5.eea4 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet5.eea5 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet5.eea6 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet5.eea7 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet6 = new com.objsys.nas.TS24501IES.UESecurityCapability_octet6();
      pvalue.replayedUECapabilities.octet6.eia0 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet6.eia1 = new Asn1Boolean(true);
      pvalue.replayedUECapabilities.octet6.eia2 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet6.eia3 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet6.eia4 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet6.eia5 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet6.eia6 = new Asn1Boolean(false);
      pvalue.replayedUECapabilities.octet6.eia7 = new Asn1Boolean(true);
      pvalue.imeisvReq = new IMEISVRequest();
      pvalue.imeisvReq.value = new Asn1Integer(1);
      pvalue.epsSecAlgorithm = new NASSecurityAlgorithms();
      pvalue.epsSecAlgorithm.typeOfCiph = NASSecurityAlgorithms_typeOfCiph.eea0();
      pvalue.epsSecAlgorithm.typeOfProt = NASSecurityAlgorithms_typeOfProt.eia0();
      pvalue.eapMessage = new EAPMessage();
      pvalue.eapMessage.length = new Asn1Integer(0); // Calculated automatically...
      pvalue.eapMessage.packetFormat = new EAPPacketFormat();
      pvalue.eapMessage.packetFormat.code = EAPFormatCode.response();
      pvalue.eapMessage.packetFormat.id = new Asn1Integer(53);
      pvalue.eapMessage.packetFormat.length = new Asn1Integer(8);
      pvalue.eapMessage.packetFormat.data = new Asn1OctetString();
      {
          byte tmpdata[] = { (byte)0xbe, (byte)0x97, (byte)0xf1, (byte)0x28 };
         pvalue.eapMessage.packetFormat.data.value = tmpdata;
      }

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
