import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded
      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.ServiceLevelAuthCmd data = new ServiceLevelAuthCmd();
      com.objsys.nas.TS24501Msgs.ServiceLevelAuthCmd pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.sessMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_sm tempVar1 = new PDU_hdrData_sm();
      pdu.hdrData.set_sm(tempVar1);

      tempVar1.pduSessIdent = PDUSessionIdentity.pduId10();
      tempVar1.procTransIdent = new Asn1Integer(120);
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.st_SvcLvlAuthCmd5G);
      pdu.data = pvalue;

      pvalue.aaContainer = new SLAAContainer();
      pvalue.aaContainer.length = new Asn1Integer(0); /* length will be calculated */

      pvalue.aaContainer.contents = new SLAAParameter[2];
      pvalue.aaContainer.contents[0] = new SLAAParameter();
      pvalue.aaContainer.contents[0].type = SLAAParamTypes.ptDeviceId();
      pvalue.aaContainer.contents[0].param = new SLAAParameter_param();

      SLDeviceId tempVar2 = new SLDeviceId();
      pvalue.aaContainer.contents[0].param.set_deviceId(tempVar2);
      tempVar2.length = new Asn1Integer(0); /* length will be calculated */
      {
         byte testdata[] = { (byte)0x80, (byte)0x54 };
         tempVar2.id = new Asn1OctetString(testdata);
      }

      pvalue.aaContainer.contents[1] = new SLAAParameter();
      pvalue.aaContainer.contents[1].type = SLAAParamTypes.ptResponse();
      pvalue.aaContainer.contents[1].param = new SLAAParameter_param();

      SLAAResponse tempVar3 = new SLAAResponse();
      pvalue.aaContainer.contents[1].param.set_response(tempVar3);
      tempVar3.length = new Asn1Integer(0); /* length will be calculated */
      tempVar3.c2ar = new Asn1Integer(3);
      tempVar3.slar = new Asn1Integer(1);

      // Create a message buffer object and encode the record
      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing
      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {
          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file
          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons
          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
