import com.objsys.asn1j.runtime.*;
import java.io.*;

import com.objsys.nas.TS24501Msgs.*;
import com.objsys.nas.TS24501IES.*;
import com.objsys.nas.TS24301IES.*;
import com.objsys.nas.TS24008IES.*;

public class Writer {


   public static void main (String args[]) {
       String filename = new String ("message.dat");
       boolean trace = true;

      // Process command line arguments

      if (args.length > 0) {
         for (int i = 0; i < args.length; i++) {
            if (args[i].equals ("-v")) 
                Diag.instance().setEnabled (true);
            else if (args[i].equals ("-o")) 
                filename = new String (args[++i]);
            else if (args[i].equals ("-notrace")) trace = false;
            else {
                System.out.println ("usage: Writer [ -v ] [ -o <filename>");
                System.out.println ("   -v  verbose mode: print trace info");
                System.out.println ("   -o <filename>  " +
                                    "write encoded msg to <filename>");
                System.out.println ("   -notrace  do not display trace info");
                return;
            }
         }
      }

      // Create a data object and populate it with the data to be encoded

      com.objsys.nas.TS24501Msgs.PDU pdu = new PDU();
      com.objsys.nas.TS24501Msgs.ULNASTrans data = new ULNASTrans();
      com.objsys.nas.TS24501Msgs.ULNASTrans pvalue = data;

      /* Populate data structure */
      pdu.protoDiscr = NAS5GProtoDiscr.mobMgmt5G();
      pdu.hdrData = new PDU_hdrData();
      PDU_hdrData_mm tempVar1 = new PDU_hdrData_mm();
      pdu.hdrData.set_mm(tempVar1);

      tempVar1.secHdrType = NAS5GSecHdrType.noSec();
      pdu.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.mt_ULNASTrans5G);
      pdu.data = pvalue;

      pvalue.containerType = PayloadContainerType.n1SmInfo();
      pvalue.container = new ULNASTrans_container();
      com.objsys.nas.TS24501Msgs.PDU_SM pdu_SM = new PDU_SM();
      com.objsys.nas.TS24501Msgs.ModifyReject data_SM = new ModifyReject();
      com.objsys.nas.TS24501Msgs.ModifyReject pvalue_SM = data_SM;

      pdu_SM.protoDiscr = NAS5GProtoDiscr.sessMgmt5G();
      pdu_SM.pduSessIdent = PDUSessionIdentity.pduId10();
      pdu_SM.procTransIdent = new Asn1Integer(5);
      pdu_SM.msgType = new _NAS5G_PROTOCOL_CLASS_msgType(_TS24501MsgsValues.st_ModifyReject5G);
      pdu_SM.data = pvalue_SM;

      N1SMPayloadContainer payload = new N1SMPayloadContainer();
      payload.length = new Asn1Integer(0); /* length will be calculated */
      payload.pduSm = new PDU_SM();
      pvalue_SM.cause = new SMCause(111);
      pvalue_SM.backoffTimer = new GPRSTimer3();
      pvalue_SM.backoffTimer.unit = GPRSTimer3_unit.valIncrMult320Hrs();
      pvalue_SM.backoffTimer.timerValue = new Asn1Integer(18);
      pvalue_SM.mV16ExtPresent = true;
      pvalue_SM.reattemptInd = new ReattemptIndicator();
      pvalue_SM.reattemptInd.eplmnc = new Asn1Boolean(false);
      pvalue_SM.reattemptInd.ratc = new Asn1Boolean(true);
      payload.pduSm = pdu_SM;
      pvalue.container.set_n1sm(payload);

      pvalue.pduSessionId = new PDUSessionId2(57);
      pvalue.oldPduSessionId = new PDUSessionId2(38);
      pvalue.requestType = new RequestType();
      pvalue.requestType.reqType = RequestTypes.emerPduSession();
      pvalue.snssai = new SNSSAI();
      pvalue.snssai.length = new Asn1Integer(63);
      pvalue.snssai.sst = new SST(120);
      pvalue.snssai.sd = new SD(13021780);
      {
          byte testdata[] = { 
         (byte)0xeb };
         pvalue.snssai.mappedSST = new Asn1OctetString(testdata);
      }
      {
          byte testdata[] = { 
         (byte)0xf1, (byte)0xb3, (byte)0xc2 };
         pvalue.snssai.mappedSD = new Asn1OctetString(testdata);
      }
      pvalue.dnn = new DNNList();
      pvalue.dnn.apn = new Asn1IA5String[3];
      pvalue.dnn.apn[0] = new Asn1IA5String("ZERO");
      pvalue.dnn.apn[1] = new Asn1IA5String("ONE");
      pvalue.dnn.apn[2] = new Asn1IA5String("TWO");
      
      pvalue.addlInfo = new AdditionalInfo();
      {
          byte testdata[] = { 
         (byte)0xd8 };
         pvalue.addlInfo.value = testdata;
      }
      pvalue.mV16ExtPresent = true;
      pvalue.maPduSessionInfo = MAPDUSessionInfo.networkUpgAllowed();
      pvalue.relAssistInd = new ReleaseAssistInd();
      pvalue.relAssistInd.ddx = ReleaseAssistInd_ddx.ddxSingleDLTrans();

      // Create a message buffer object and encode the record

      Asn1NasEncodeBuffer encodeBuffer = new Asn1NasEncodeBuffer();

      // Enable bit field tracing

      if (trace) {
         encodeBuffer.getTraceHandler().enable();
      }  

      try {

          pdu.encode (encodeBuffer);

          if (trace) {
              System.out.println ("Encoding was successful");
              System.out.println ("Hex dump of encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump:");
              encodeBuffer.binDump ("message");
          }

          // Write the encoded record to a file

          encodeBuffer.write (new FileOutputStream (filename));

          // Generate a dump file for comparisons

          encodeBuffer.hexDump 
              (new PrintStream (new FileOutputStream ("message.dmp")));
      }
      catch (Exception e) {
          if (trace) {
              System.out.println ("Hex dump of partially encoded record:");
              encodeBuffer.hexDump ();
              System.out.println ("Binary dump of partially encoded record:");
              encodeBuffer.binDump ("message");
          }

          System.out.println (e.getMessage());
          e.printStackTrace();           
          System.exit(-1);
      }
   }
}
